;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                   ;
;      FLOPPY  MOPPEL-FDC-Utilities  V 10.6         ;
;                                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;   Stand:   Mo,01.06.87;12:06:42h
;
;
; HMS Quellen Ueberarbeitet und Anpassung auf 8085Simulator-ID
;			https://www.oshonsoft.com/
;			notepad++ Tab = 6
;			werner.roemer@t-online.de
;
; 30.07.2023 	externe Bereiche ans Ende gestellt
;			(Vergleich EPROM mit Quelle ist OK)
;

;
;----------------------------------------------------------
;
	org    2000h ;Hilfsbank 0 (=00h OUT $40h)
;
;
;      Z80-Kompatibilit{t
;      (Laden des Interrupt-Masken-Registers)
;
;  TRAP  (24/66h): PCHL        <E9>        DRQ
;  RST5.5 (002Ch): POP  PSW    <F1>      INTRQ
;         (2F98h)  IN   STATUS <DB 48h>
;                  RET         <C9>
;
;
;  Block schreiben:2006h; Block lesen:2009h
;
;  Parameter-]bergabe:
;
;  LWNR:    Laufwerk-Nr.0...3   (=A...D)
;  SIDNR:   Seite       0/1
;  TRKNR:   Spur        0...4Fh (0...79d)
;  SECNR:   Sektor      0...0Fh (0...16d)
;  STEP:    Step-Rate   0...3   (6/12/20/30 ms)
;  RAMBEG:  RAM-Beginn  0...FF00h
;  RAMEND:  RAM-Ende    0...FFFFh
; (DENSY:   DD/SD       0/1;     double/single)
; (SECZAL:  Sec.p.Trk.  1...10h; 1...16d)
; (SECL:    Bytes p.Sec.0...3;   128/256/512/1024d)
;
;
;  SELOUT($40h) 7  6   5   4  3  2  1   0
;               ;  ;   ;   ;  ;  ;  ;   ;
;               !  !   !   !  !  !  !   0 = DD
;               !  !   !   !  !  !  !   1 = SD
;               !  !   !   !  !  !  1   . = Drive A
;               !  !   !   !  !  1  .   . = Drive B
;               !  !   !   !  1  .  .   . = Drive C
;               !  !   !   1  .  .  .   . = Drive D
;               !  !   0   .  .  .  .   . = Side 0
;               !  !   1   .  .  .  .   . = Side 1
;               x  x   .   .  .  .  .   . = n.c.
;
;
;  STATUS($48h) 7   6   5   4   3   2   1  0
;               ;   ;   ;   ;   ;   ;   ;  ;
;               !   !   !   !   !   !   x  x = n.c.
;               !   !   !   !   !   1 = Lost Data
;               !   !   !   !   1 = CRC Error
;               !   !   !   1 = Record Not Found
;               !   !   1 = Record Type/Spin-up
;               !   1 = Write Protect
;               1 = Motor On
;
;
; WD1770-Befehle:
;
restor	equ	 00h  ;Pos.Track 00
seek  	equ	 10h  ;Pos.Track tt
stepin	equ	 50h  ;Pos.Track tt+1
rdsc  	equ	 80h  ;Read Sector ss
wrsc  	equ	0a2h  ;Write Sector ss (no Precomp.)
forst 	equ	0d0h  ;Force reset
force 	equ	0d8h  ;Force Immediate Interrupt
wrtrk 	equ	0f2h  ;Write Track
;
;
timout 	equ	 0ah ;10d x 0,5 s
;      	equ
stepff 	equ	1ah  ;Portadressen
;      	equ
vidctl 	equ	2ah
;      	equ
ctrl   	equ	40h  ;;'374
;      	equ
COMREG 	equ	48H  ;WD1770-Regs
TRKREG 	equ	49H
SECREG 	equ	4AH
DATREG 	equ	4BH
STATUS 	equ	COMREG
;
;
dely1  	equ	0bh  ;Monitor-Unterprogramme
CO     	equ	49H
;
covid  	equ	1006h
consin 	equ	1009h
ci     	equ	consin
cstats 	equ	100ch
csts   	equ	cstats
PARIN  	equ	1012H
STRING 	equ	1021H
error-	equ	1027h
clrlin 	equ	1030h
crspos 	equ	1033h
CRSON  	equ	1036H
CRSOFF 	equ	1039H
ok     	equ	1048h
;
movid  	equ	1003h  ;externe Sprungziele
ass    	equ	6000h
edi    	equ	7000h
;
batbuf 	equ	2d00h  ;RAM-Zellen
fdcbuf 	equ	2e90h  ;Ende LINBUF
;
linlen 	equ	2e4ch  ;Zeilenl{nge
hordsp 	equ	2e4eh
liny   	equ	2e52h
linz   	equ	2e54h
update 	equ	2e56h
STACK  	equ	2F00H
RST55  	equ	2F98H
RST551 	equ	2F99H
srcbeg 	equ	2fbfh
srcend 	equ	2fc1h
dstbeg 	equ	2fc3h
;
;
FLOPPY:	JMP  FKALT     ;Einsprungleiste
		jmp  warmex
wrblok:	jmp  wrct      ;Block schreiben
rdblok:	jmp  rdct      ;Block lesen
batch0:	jmp  batklt    ;Batch-0-Schnellstart
rdsec: 	push h         ;V.5-CP/M
		mov  a,e
		mov  h,a
		mov  l,d
		shld trknr
		pop  h
		mov  a,h
		sta  rambeg
		mvi  a,rdsc
		sta  wdbef
		call rdwr
		ani  5ch
		ret
;
;
kdens: db  00h        ;Kaltstart-Parameter;DD
ksecs: db  10h        ;16d Secs.p.Trk.
ksecl: db  01h        ;256 Bytes/Sec.
kstep: db  03h        ;Step-Rate
kramb: db  90h        ;RAM-Beginn 9000h
krame: db 0EFh        ;RAM-Ende EFFFh
ktrkz: db  28h        ;Spurzahl p.Disk.
;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Randbedingungen herstellen
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
setup: 	mvi  a,0c3h    ;JMP
		sta  rst55
		lxi  h,isr
		shld rst551
;
		mvi  a,1eh     ;RST5.5/RSTC enable
		mvi  b,2h
		sim
		nop
		mov  a,b
		out  0bbh
;
		call crsoff    ;Cursor aus
		lhld rambeg
		mov  a,h
		sub  l
		mov  h,a
		mvi  a,97h
		jc   errout
		mov  a,h
		inr  a
		sta  reczal    ;Records
;
		lda  lwnr
		mov  b,a
		mvi  a,1h
selop: 	rlc
		dcr  b
		jp   selop
		mov  b,a
		lda  densy
		ora  b
		mov  b,a
		lda  sidnr
		rrc
		rrc
		rrc
		ora  b
		sta  selout    ;CTRL-Reg.'374
;
		lxi  h,drivea
		lda  lwnr
		mov  e,a
		mvi  d,0h
		dad  d
		shld drvpoi    ;LW-Anwahl
;
		lda  secl
		lxi  h,80h
seclop:	shld seclen    ;Sektor-L{nge
		ora  a
		rz
		dad  h
		dcr  a
		jmp  seclop
;
;
inibat:	lhld rambeg
		shld rambuf
		lxi  h,2d2dh   ;Batch-Init.
		shld rambeg
ininul:	xra  a         ;Init.
		out  stepff
		sta  lwnr
		sta  sidnr
		sta  trknr
		inr  a
		sta  secnr
		ret
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Record lesen und schreiben
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
rdwr:  	call posit     ;Kopf pos.
		lda  secnr
		out  secreg
		lda  wdbuf     ;R/W-Befehl holen
		push psw
		call cmdsp     ;anzeigen
		lda  rambeg
		mov  d,a
		mvi  e,0h
		lhld intbuf
		pop  psw
		out  comreg    ;und laden
rdwrlp:	ei             ;Schreib/Lese-Schleife
		hlt
		jmp  rdwrlp
;
intrd: 	in   datreg    ;DRQ>TRAP>PCHL
		stax d
		inx  d
		ret
;
intwr: 	ldax d         ;DRQ>TRAP>PCHL
		out  datreg
		inx  d
		ret
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Spur schreiben (Formatieren)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
formdd:	lda  wdbuf
		mvi  b,1h      ;Sektorz{hler
		lxi  d,1000h   ;16d Secs/256d Bytes
		lhld intbuf
		out  comreg
		ei
trknew:	mvi  a,4eh     ;Spurbeginn
		mvi  c,3ch     ;60d
lop0:  	hlt
		dcr  c
		jnz  lop0
;
secnew:	mvi  c,0ch     ;12d;Sektorbeginn
		xra  a
lop1:  	hlt
		dcr  c
		jnz  lop1
;
		mvi  a,0f5h    ;;=A1h!
		hlt
		hlt
		hlt
;
		mvi  a,0feh    ;Ind.Addr.Mark
		hlt
;
		lda  trknr     ;Trk.Nr.
		hlt
;
		lda  sidnr     ;Side
		hlt
;
		mov  a,b       ;Sec.Nr.
		hlt
		inr  b
;
		lda  secl      ;Len.
		hlt
;
		mvi  a,0f7h    ;2xCRC
		hlt
;
		mvi  a,4eh     ;Gap 2
		mvi  c,16h     ;22d
lop2:  	hlt
		dcr  c
		jnz  lop2
;
		xra  a
		mvi  c,0ch     ;12d
lop3:  	hlt
		dcr  c
		jnz  lop3
;
		mvi  a,0f5h    ;;=A1h!
		hlt
		hlt
		hlt
;
		mvi  a,0fbh    ;Dat.Addr.Mark
		hlt
;
		mvi  a,0e5h    ;Datenblock
lop4:  	hlt
		dcr  e
		jnz  lop4
;
		mvi  a,0f7h    ;2xCRC
		hlt
;
secend:	mvi  a,4eh     ;Sektorende; Gap 3
		mvi  c,36h     ;54d;schneller als 24d!
lop5:  	hlt
		dcr  c
		jnz  lop5
		dcr  d
		jnz  secnew
;
trkend:	hlt            ;Spurende
		ei             ;;!
		jmp  trkend
;
intfor:	out  datreg    ;DRQ>TRAP>PCHL
		ret
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      WD1770-Ansteuerung
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
setrk0:	xra  a         ;Spur 0 einstellen
		sta  trknr
		mvi  a,restor
		call cmdbot    ;Befehl anzeigen und ausgeben
		ani  18h
		jnz  error
		jmp  pard      ;Kopfzeilen aufdatieren
;
posit: 	lhld drvpoi    ;Ziel in TRKNR
		push h
		lda  wdbef
		sta  wdbuf
		lda  trknr
		push psw
		mov  a,m
		ora  a
		lda  trknr
		push psw
		cp   setrk0    ;Nr.ung}ltig:neu pos.
		pop  psw
		sta  trknr
		pop  psw
		pop  h
		mov  b,m
		ori  80h
		mov  m,a       ;Nr.ins Disp.
		ani  7fh
		jz   setrk0
		mov  c,a
		out  datreg    ;gew.Pos.
		mov  a,b
		ani  7fh
		out  trkreg    ;akt.Pos.
		cmp  c
		rz             ;ges=akt.Pos.:RET!
		mvi  a,seek
		call cmdbot
		ani  18h
		jnz  error
precmp:	lda  densy     ;Precomp.testen
		ora  a
		rnz            ;SD:RET
		lda  trknr
		cpi  2ah
		rc             ;kleiner 42d:RET
		lxi  h,wdbuf
		mov  a,m
		ani  0fdh      ;ab Trk.42d ohne Precomp.
		mov  m,a
		ret
;
cmdbot:	mov  b,a       ;Befehl ausgeben
		lda  step
		ora  b
cmdout:	call dely1     ;wg.langs.LW
		call pardsp
		out  comreg    ;erst WD1770 (wg.87er-HW)
moton: 	in status
		rlc            ;und Motor On abwarten
		jnc  moton
		lda  selout
		out  ctrl      ;dann '373/'174
		ei
;
		mvi  c,timout
timlop:	lxi  d,0h      ;0,5 s
timlp: 	dcx  d
		mov  a,d
		ora  e
		jnz  timlp
		dcr  c
		jnz  timlop
		mvi  a,99h     ;Timout-Err.
		jmp  errout
;
error:	 mvi  c,8h
errcod:	dcr  c
		rlc
		jnc  errcod
		mov  a,c
		adi  90h
errout:	di
		call error
;
		mvi  a,force
		call cmdout
		lhld drvpoi
		mvi  m,0h
		call pard
		mvi  a,forst
		out  comreg
		jmp  warm
;
isr:   	pop  psw       ;clear RET-Adr.
		in   status
		ret
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Hauptprogramm
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
batklt:	mvi  a,"d"
		jmp  fklt
fkalt: 	xra  a
fklt:  	sta  cpmflg
		lxi  h,0h
		shld drivea
		shld drivec
		call ininul    ;LW/Sid/Trk/Sec
		lhld kdens
		shld densy     ;Dens/Secs
		lhld ksecl
		mov  a,l
		sta  secl
		mov  a,h
		sta  step
		call setup
;
warmex:	lxi  h,head
		call string    ;Kopfzeilen
		lhld kramb
		xchg
		lda  ktrkz
		call bindec
		mov  b,a
		lxi  h,mspar1
		mvi  a,9h      ;;=GET3
		call getpar    ;RAMb/RAMe/Trks
		call decbin
		sta  trkzal
		shld rambeg
warm0: 	xra  a
warm:  	call pardp
		lxi  sp,stack
		lxi  h,retur
		push h
		lhld liny
		call crspos
		lxi  h,prompt
		call strip     ;F>
		call crson
		lda  cpmflg
		ora  a
		jnz  wrm       ;Schnellstart bei FLAG="d"
		call ci
		cpi  03h       ;CTL+C
		jz   warmex
		cpi  1bh       ;ESC
		jz   warmex
wrm:   	MOV  C,A
		CALL CO
		ANI  5FH
		CPI  "A"
		JZ   ASS
		CPI  "B"
		JZ   BATCH
		CPI  "C"
		JZ   COPY
		CPI  "D"
		JZ   DISKIN
		CPI  "E"
		JZ   EDI
		CPI  "F"
		JZ   FORMAT
		CPI  "M"
		JZ   MOVID
		CPI  "R"
		JZ   READ
		CPI  "W"
		JZ   WRITE
		mvi  a,90h
		jmp  errout
retur: 	call ok
		lda  wdbef
		jmp  warm
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Batch-Vorspann ausgeben
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
batch: 	lxi  h,msbatc
		mvi  d,0h
		mvi  b,28h
		mvi  a,6h      ;;=GET2
		call getpar    ;Nr/Start
		push h
		mov  a,l
		call decbin
		rlc
		rlc
		rlc
		mov  l,a
		mvi  h,2dh     ;H&L:Batch-Pointer
		push h
		call dskrd     ;Trk0/Sec1
		pop  h
		pop  b
		mvi  m,0h      ;Startadr.(LO)
		inx  h
		mov  m,b       ;Startadr.(HI)
		inx  h
		push h
		call rdsub     ;LW/Trk/Sec
		call setup     ;wg.Recs.
		pop  h
		lda  reczal
		mov  m,a       ;Records
		inx  h         ;xx (Step)
		inx  h
		lda  secnr
		mov  m,a       ;Sektor
		inx  h
		lda  trknr
		mov  m,a       ;Spur
		inx  h
		mvi  m,0h      ;RAM-Adr.(LO)
		inx  h
		lda  rambeg
		mov  m,a       ;RAM-Adr.(HI)
		call batwr     ;Trk0/Sec1
		jmp  parend
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Spuren kopieren
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
copy:  	lda  trkzal    ;Spur-Anzahl
		push psw
		lhld rambeg
		push h
		mvi  a,0fh     ;16 Recs.
		add  l
		sta  ramend
;
		call rdsub     ;Src-Params.
		lxi  h,srcpar
		call cpysav
		call wrsub     ;Dst-Params.
		lxi  h,dstpar
		call cpysav
;
cpylop:	lxi  h,srcpar
		push h
		call cpyrd     ;Spur lesen
		pop  h
		inr  m         ;Src-Trk
		lxi  h,dstpar
		push h
		call cpywr     ;Spur schreiben
		pop  h
		inr  m         ;Dst-Trk
		lxi  h,trkzal
		dcr  m
		jnz  cpylop
		pop  h         ;urspr.Params
		shld rambeg
		pop  psw
		sta  trkzal
		jmp  parend
;
;
cpysav:	lda  secnr     ;Save Params.
		mov  d,a
		lda  lwnr
		mov  c,a
		lda  sidnr
		mov  b,a
		lda  trknr
		mov  m,a
		inx  h
		mov  m,b
		inx  h
		mov  m,c
		inx  h
		mov  m,d
		ret
;
cpyrst:	mov  a,m       ;Restore Params.
		sta  trknr
		inx  h
		mov  a,m
		sta  sidnr
		inx  h
		mov  a,m
		sta  lwnr
		inx  h
		mov  a,m
		sta  secnr
		ret
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Block einlesen und Programmstart (Batch)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
diskin:	lxi  h,msdisk
		mvi  b,0h
		call gtpar1    ;;=GET1; Nr.
		mov  a,l
		call decbin
		rlc
		rlc
		rlc            ;L=8;L
		mov  l,a
		mvi  h,2dh     ;H&L:Batch-Pointer
		push h
		call dskrd     ;Trk0/Sec1
		pop  h
		mov  e,m       ;Startadr.(LO)
		inx  h
		mov  d,m       ;Startadr.(HI)
		inx  h
		mov  c,m       ;Records
		mov  b,d
		push b
		inx  h         ;xx (Step)
		inx  h
		mov  a,m       ;Sektor
		sta  secnr
		inx  h
		mov  a,m       ;Spur
		sta  trknr
		inx  h         ;xx(RAM-Adr.<LO>)
		inx  h
		mov  a,m       ;RAM-Adr.(HI)
		sta  rambeg
		pop  h
		add  l
		dcr  a
		sta  ramend
		xra  a
		sta  cpmflg    ;Schnellstart-FLAG l|schen
		mvi  l,0h
		push d
		call rdct      ;Programm einlesen
		pop  h
		pchl           ;und starten
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Diskette formatieren
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
format:	mvi  b,0ah
		lxi  h,msform
		call gtpar1    ;;=GET1; LW
		mov  a,l
		call drivin
		lxi  h,intfor
		shld intbuf
		call setup
		call setrk0
		mvi  a,wrtrk
		sta  wdbuf
formlp:	call formdd    ;Spur schreiben
		ani  58h
		jnz  error
		lxi  h,trknr
		inr  m
		lda  trkzal
		dcr  a
		cmp  m
formnd:	jc  setrk0
		mvi  a,stepin  ;Kopf weiter
		call cmdbot
		mvi  a,wrtrk
		sta  wdbuf
		call precmp    ;Precomp.testen
		lda  wdbuf
		call cmdsp     ;Befehl anzeigen
		jmp  formlp
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Block lesen und schreiben
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
read:  	call rdsub
		jmp  rdwcte
;
write: 	call wrsub
		jmp  rdwcte
;
rdsub: 	mvi  a,rdsc    ;Params.einlesen
		lxi  d,msread
		lxi  h,intrd
		jmp  rdwsub
wrsub: 	mvi  a,wrsc
		lxi  d,mswrit
		lxi  h,intwr
rdwsub:	sta  wdbuf
		shld intbuf
		xchg
		lxi  d,0ah
		mvi  b,1h
		mvi  a,9h      ;;=GET3
		call getpar    ;LW/Trk/Sec
		call decbin
		sta  secnr
		mov  a,h
		call decbin
		sta  trknr
		mov  a,l
		call drivin
		lhld rambeg
		shld rambuf
		ret
;
cpyrd: 	call cpyrst    ;Read-Einspr}nge
		jmp  rdct
dskrd: 	call inibat
rdct:  	mvi  a,rdsc
		lxi  h,intrd
		jmp  rwct
;
cpywr: 	call cpyrst    ;Write-Einspr}nge
		jmp  wrct
batwr: 	call inibat
wrct:  	mvi  a,wrsc
		lxi  h,intwr
rwct:  	shld intbuf
		sta  wdbuf
;
rdwcte:	call setup     ;RD/WR-Unterprogramm
rwlop: 	lda  wdbuf
		sta  wdbef
		mvi  c,5h
rwdh:  	push b
		call rdwr
		pop  b
		ani  5ch
		jz   rwcte
		dcr  c
		jnz  rwdh
		inr  a         ;ZERO off
		jmp  rdwend
rwcte: 	lxi  h,reczal
		dcr  m
		jz   rdwend
		lxi  h,rambeg
		inr  m
		lxi  h,secnr
		inr  m
		lda  seczal
		cmp  m
		jnc  rwlop
		mvi  m,1h
		lxi  h,trknr
		inr  m
		jmp  rwlop
rdwend:	push psw
		lhld rambuf
		shld rambeg
		pop  psw
		jnz  error
		jmp  parend
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Parameter-Eingabe
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
gtpar1:	mvi  a,3h      ;;=GET1
getpar:	push b
		push d
		push psw
		sta  parlen
		call strip     ;Textzeile ausgeben
		lhld update
		pop  psw
		pop  d
		pop  b
;
		cpi  3h
		jz   get1
		cpi  6h
		jz   get2
get3:  	mov  a,e
		call bytkom
get2:  	mov  a,d
		call bytkom
get1:  	mov  a,b
		call bytot
		shld hibuf     ;Pos.hinter Params merken
		call crson
;
getest:	lda  cpmflg
		ora  a
		jz   getlop
		mvi  a,0dh
		jmp  getret    ;Schnellstart
getlop:	call ci
		cpi  03h
		jz   warm0
		cpi  1bh
		jz   warm0
getret:	mov  c,a
		call co
		cpi  0dh
		jnz  getlop
		lhld hibuf
		lda  parlen
		mov  c,a
		call parin     ;Params.vom Schirm einlesen
		mvi  a,91h
		jz   errout
		lda  srcbeg
		mov  e,a
		lda  srcend
		mov  d,a
		xchg
		lda  dstbeg
		ret
;
;
drivin:	push psw       ;Seite/Laufwerk trennen
		ani  10h
		rrc
		rrc
		rrc
		rrc
		sta  sidnr
		pop  psw
		ani  0fh
		sui  0ah
		sta  lwnr
		ret
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Video-Ausgabe
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
posln5:	lxi  h,lin5    ;zur Zeile 5
		jmp  string
;
strip: 	mov  a,m       ;STRING mit festst.Kopfzeilen
		ora  a
		rz
		inx  h
		mov  c,a
		call co
		cpi  0dh
		jnz  strip
		push h         ;Save&Scroll
		lhld update
		push h
		mvi  c,1bh
		call co
		mvi  c,14h
		call co
		call posln5
		lda  hordsp
strlop:	mvi  c,"-"     ;Trennstrich
		call co
		dcr  a
		jnz  strlop
		lhld linz
		call clrlin    ;unterste Zeile l|schen
		pop  h
		call crspos
		pop  h
		jmp  strip
;
crlf:  	lxi  h,crlfms
		jmp  string
crlfms:	db   0dh
		db   0h
;
parend:	lda  wdbuf
pardsp:	push psw       ;Kopfzeilen aufdatieren
		lhld drvpoi
		lda  trknr
		ori  80h
		mov  m,a
		pop  psw
pardp: 	sta  wdbef
pard:  	push psw
		lxi  h,3000h
		shld hibuf
;
		lda  drivea    ;Zeile 1
		call drvout
		lda  sidnr
		mov  b,a
		lda  step
		rlc
		rlc
		rlc
		rlc
		ora  b
		call binout
		lda  densy
		call densot
;
		lda  trkzal    ;Zeile 2
		call bindec
		mov  b,a
		lda  seczal
		call bindec
		mov  c,a
		lda  driveb
		call parsub
;
		lda  secl      ;Zeile 3
		mov  c,a
		lda  rambeg
		mov  b,a
		lda  drivec
		call parsub
;
		lda  wdbef     ;Zeile 4
		mov  c,a
		lda  ramend
		mov  b,a
		lda  drived
		call parsub
;
		pop  psw
		ret
;
parsub:	push b
		push psw
		lhld hibuf
		xchg
		lhld linlen
		dad  d         ;;=CRLF
		shld hibuf
		pop  psw
		call drvout
		pop  b
		mov  a,b
		push b
		call binout
		pop  b
		mov  a,c
		jmp  binout
;
;
cmdsp: 	lhld cmdpoi    ;nur Befehl anzeigen
		push psw
		call bytot
		pop  psw
		ret
;
drvout:	call next      ;Kopf-Position anzeigen
		ora  a
		push psw
		ani  7fh
		mov  b,a
		pop  psw
		mov  a,b
		jm   decot
		mvi  c,"x"
		call covid
		jmp  covid
;
densot:	call next2     ;Schreibdichte anzeigen
		mvi  c,"D"
		ora  a
		jz   covid
		mvi  c,"S"
		jmp  covid
;
decot: 	call bindec    ;Dezimalanzeige
		jmp  bytot
;
bytkom:	call bytot     ;Byte plus Komma
		mvi  c,","
		jmp  covid
;
binout:	call next2     ;Bin{ranzeige
;
bytot: 	push psw       ;Byte anzeigen
		rrc
		rrc
		rrc
		rrc
		call nibot
		pop  psw
;
nibot: 	ani  0fh       ;Nibble anzeigen
		ori  30h
		cpi  3ah
		jc   nibcte
		adi  7h
nibcte:	mov  c,a
		jmp  covid
;
bindec:	mov  c,a       ;Bin{r/Dezimal-Umsetzung
		ora  a
		rz
		xra  a
binlop:	inr  a
		daa
		dcr  c
		jnz  binlop
		ret
;
decbin:	ora  a         ;Dezimal/Bin{r-Umsetzung
		rz
		mov  b,a
		xra  a
		mov  c,a
declop:	ora  a         ;Clear AC-FLAG
		inr  c
		inr  a
		daa
		cmp  b
		jnz  declop
		mov  a,c
		ret
;
next2: 	call next      ;Kopf-Tabs
next:  	lxi  d,6h
		dad  d
		shld cmdpoi
		ret
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Klartext-Meldungen
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
head:  	db 0ch ;Clear Screen
		db 1bh ;Home
		db 11h
		db "Drv.A:xx    Stp/Sid:xx       Dens:xD"
		db 0dh
		db "Drv.B:xx    Tracks: xxd      Secs:xxd"
		db 0dh
		db "Drv.C:xx    RAM-Beg:xxh      Byts:xxh"
		db 0dh
		db "Drv.D:xx    RAM-End:xxh      Comd:xxh"
		db 1bh
		db 17h
		db 0dh
		db 0dh
		db 0dh
		db 0dh
		db 0dh
		db 0dh
		db 0dh
		db "MOPPEL-FDC-Utilities V 10.6"
		db 0dh
		db "Copyright (C) hms'86"
		db 0dh
		db 0dh
		db " A/E: Assembler/Editor"
		db 0dh
		db " B/D: Batch out/Disk in"
		db 0dh
		db " C/F: Copy/Format"
		db 0dh
		db " R/W: Read/Write"
		db 0dh
		db "  M:  Monitor"
		db 1bh
		db 16h
		db 04h
		db 27h
		db 0dh
		db 0dh
		db 0dh
		db 0dh
		db 0h
;
prompt:	db 0dh
		db 0dh
		db "F>"
		db 0h
;
mspar1:	db 0dh
		db "RAMb,RAMe,Tracks>"
		db 0h
;
msread:	db 0dh
		db "Src-Drv,Trk,Sec>"
		db 0h
;
mswrit:	db 0dh
		db "Dst-Drv,Trk,Sec>"
		db 0h
;
msdisk:	db 0dh
		db "Num>"
		db 0h
;
msbatc:	db 0dh
		db "Num,Start>"
		db 0h
;
msform:	db 0dh
		db "Drv>"
		db 0h
;
lin5:  	db 1bh
		db 16h
		db 04h
		db 27h
		db 0dh
		db 0h
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      RAM-Buffer
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
	ORG     FDCBUF
;
densy: 	ds 1h ;Schreibdichte
seczal:	ds 1h ;Secs.p.Trk.
secl:  	ds 1h ;Sektor-L{nge (cod.)
seclen:	ds 2h ;Sektor-L{nge (bin.)
wdbef: 	ds 1h ;WD1770-Befehl
wdbuf: 	ds 1h ;Buffer f.WDBEF
cmdpoi:	ds 2h ;DSP-Pointer f.Bef.
;
sidnr: 	ds 1h ;Seite
step:  	ds 1h ;Step-Rate
trkzal:	ds 1h ;Trks.p.Disk.
rambeg:	ds 1h ;RAM-Buffer (Start)
ramend:	ds 1h ;RAM-Buffer (Ende)
rambuf:	ds 2h ;Buffer f.RAMBEG/RAMEND
reczal:	ds 1h ;Records
;
drivea:	ds 1h ;Kopf-Pos.
driveb:	ds 1h
drivec:	ds 1h
drived:	ds 1h
drvpoi:	ds 2h ;LW-Pointer
;
lwnr:  	ds 1h ;LW-Nr.
trknr: 	ds 1h ;Spur-Nr.
secnr: 	ds 1h ;Sektor-Nr.
selout:	ds 1h ;CTRL-Reg.'374
;
srcpar:	ds 4h ;Src-Trk/Sid/Drv/Sec
dstpar:	ds 4h ;Dst-Trk/Sid/Drv/Sec
;
parlen:	ds 1h ;Params-L{nge
intbuf:	ds 2h ;Buffer f.Interrupt-Sprungadresse
hibuf: 	ds 2h ;Zwischenspeicher
cpmflg:	ds 1h ;Batch-0-Schnellstart
;
;
END
;