;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                   ;
;      HEXMO  MOPPEL-HEX-Monitor  (Symbole)  V 5.6  ;
;                                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;   Stand:   Do,19.02.87;08:37:56h
;
; HMS Quellen Ueberarbeitet und Anpassung auf 8085Simulator-ID
;			https://www.oshonsoft.com/
;			notepad++ Tab = 6
;			werner.roemer@t-online.de
;
; 30.07.2023 	externe Bereiche ans Ende gestellt
;			(Vergleich EPROM mit Quelle ist OK)
;
;
;----------------------------------------------------------
;
;
; Gilt fuer alle uebrigen PROMO-Programmteile: HEXMO,
; STP/RAM, EZU/DSP sowie Peripherie-Verwaltung
; Zum Assemblieren eines dieser Programmteile wie
; folgt vorgehen:
;
; 1. Programmteil nach 9000 ff. ins RAM laden, mit
;    Editor bearbeiten und auf Diskette abspeichern
; 2. Basis-Symboltabelle nach 9000 ff. ins RAM laden
;    und Assembler-Durchlauf 1 und 2 starten.
; 3. Quell-Listing von 1. nach 9000 ff. ins RAM
;    laden und Assembler-Durchlaeufe 0 und 2
;    starten (an vorhandene Symboltabelle anhaengen).
; 4. Bei Bedarf Assembler-Durchlauf 3 starten und/
;    oder Objektcode aus F000 ff. ins EPROM laden.
;
;----------------------------------------------------------
;
ledpor 	equ  0h        ;LED Port
prmled 	equ  0ah       ;PRM-LEDs
prmctl 	equ  0bh       ;PRM Control
ezudat 	equ  18h       ;EZU-Daten
ezuctl 	equ  19h       ;CTL EZU
stpff  	equ  1ah       ;STP-Flipflop
prtmot 	equ  38h       ;Thermo-Motor
datreg 	equ  4bh       ;FDC-Datenregister
fdctrl 	equ  40h       ;FDC/Adre~latch
outpor 	equ  80h       ;Digit 7 (+10h)
sermod 	equ  0a9h      ;Mode-Wort SER-IF
serctl 	equ  0e9h      ;SIO-CTL
serdat 	equ  0f9h      ;SIO-Daten
timctl 	equ  0d9h      ;Timer-CTL
timdat 	equ  89h       ;Timer-Daten
censtb 	equ  0bah      ;Centronics-CTL
cendat 	equ  0cah      ;Centronics-Daten
belout 	equ  0b9h      ;Schnarre
parpoa 	equ  0a8h      ;PIO-Port A
parpoc 	equ  0c8h      ;PIO-Port C
parctl 	equ  0d8h      ;PIO-CTL
hexlo  	equ  0ech      ;HEX 0...7
hexhi  	equ  0dch      ;HEX 8...F
comand 	equ  0bch      ;CMD-Keys
;
nxtnr  	equ  0h        ;NXT-Spalte
fctnr  	equ  2h        ;FCT-Spalte
adrnr  	equ  4h        ;ADR-Spalte
runr   	equ  6h        ;RUN-Spalte
diodnr 	equ  7h        ;Ge-Diode
;
rst5on 	equ  1eh       ;8085
rst6on 	equ  1dh
rst7on 	equ  1bh
rstoff 	equ  17h
;
romext 	equ  1000h     ;CPU-ROM (upper)
rambeg 	equ  2800h     ;CPU-RAM
ramend 	equ  2e00h     ;Ende Anwender-Bereich
stack  	equ  2f00h     ;entspr. MOVID
usrstk 	equ  2f70h     ;Anwender-Stack
;
uhrbuf 	equ  2fdch     ; BCD-Uhrzeit-Buffer
;
;ascbuf 	equ  2feah     ; ASCII-Uhrzeit-Buf.
;
;----------------------------------------------------------
;
	org    0000h

;
;
kalt:  	jmp  0d0h ;Kaltstart
warm:  	jmp  0f7h ;Warmstart
fquarz:	db     6h ;Quarzfrequenz in MHz
timprl:	db    10h ;Key-Prellzeit in ms
rst1:  	jmp 0a9eh ;Breakpoint setzen
dely1: 	jmp  4afh ;1 ms
v24bdr:	ds     0h ;V.24=PRT-Baudrate
prtbdr:	dw  4800h ;PRT-Baudrate
rst2:  	jmp 0b41h ;Breakpoint l|schen
dly100:	jmp  48bh ;100 ms (mit Anzeige)
casbdr:	dw  1200h ;CAS-Baudrate
rst3:  	jmp  int3
onesec:	jmp  47fh ;1 s (mit Anzeige)
dfuebdr:	dw  4800h ;DF]-Baudrate (4800/16 = 300 Bd)
;
rst4:  	jmp  int4
		db   0h
trap:  	pchl           ;FDC
		jmp  intrap
rst5:  	jmp  int5
		db   0h
rst55: 	jmp  int55     ;FDC
		db   0h
rst6:  	jmp  int6
		db   0h
rst65: 	jmp  int65     ;EZU
		db   0h
rst7:  	jmp  int7
		db   0h
rst75: 	jmp  int75     ;DF]
		db   56h       ;HEX-Monitor Ver. 5.6
;
adrset:	jmp   370h ;Adresse laden
ci:    	jmp  184bh ;Console In (ohne CSTS off)
ri:    	jmp   882h ;Reader In (Cass.)
co:    	jmp  1590h ;Console Out (Video)
po:    	jmp   89bh ;Puncher Out (Casas.)
lo:    	jmp   8b3h ;Lister Out (Drucker)
csts:  	jmp  1977h ;Cons.Status (Keyon bei FCTCNT on)
csinit:	jmp   6c6h ;CAS-IF init
loinit:	jmp   6a2h ;PRT-IF init
maxram:	jmp  0d42h ;Memory Top
v24mod:	ds      0h ;8 Bits, je 1 Start u.Stop
prtmod:	db     14h
casmod:	db     14h
dfuemod:	db     95h ;8 Bits, je 1 Start u.Stop, CLK/16
time:  	jmp   5e7h ;Uhrzeit/Datum-String
v24ini:	dw    6d4h ;V24-Init. (Adresse!)
		pchl       ;NSC800-TRAP
cnvbyt:	jmp   296h ;SSG-Umsetzung
reles: 	jmp   44eh ;Loslassen abwarten
press: 	jmp   469h ;Druecken abwarten
hhkey: 	jmp   183h ;HEX-Input
cckey: 	jmp   32fh ;CMD-Input
datset:	jmp   360h ;Daten-Mode einstellen
writur:	jmp   4d2h ;EZU stellen
readur:	jmp   533h ;EZU lesen
save:  	jmp   14fh ;Regs retten
restor:	jmp   421h ;Regs laden, Start
fill:  	jmp  0cc2h ;Konst. ins RAM
copy:  	jmp  0c75h ;RAM kopieren
casot: 	jmp   765h ;CAS-Ausgabe
casin: 	jmp   7b8h ;CAS-Eingabe
exchan:	jmp  0bc0h ;COPY mit Adre~umrechnung
ezudsp:	jmp   5b8h ;EZU in SSG-Anzeige
dspacc:	jmp   65eh ;(ACC) darstellen
dsph&l:	jmp   678h ;(H&L) darstellen
dsperr:	jmp   68eh ;Err-Anzeige
;
fctab: 	ds      0h ; FCT-Sprungverteiler
		jmp   75fh ; FCT0: CAS-Ausgabe
		jmp   7abh ; FCT1: CAS-Eingabe
		jmp   972h ; FCT2: CPU-SOD-Ausgabe (CPUOT-3)
		jmp   98dh ; FCT3: CPU-SID-Eingabe (CPUIN-3)
		jmp  0a02h ; FCT4: HEX-Dump (Thermodr.)
		jmp  0a15h ; FCT5: ASCII-Dump (Thermodr.)
		jmp  0bb3h ; FCT6: Exchange
		jmp  0c25h ; FCT7: Insert
		jmp   4c0h ; FCT8: Writur
		jmp   52dh ; FCT9: Readur
		jmp  0cb0h ; FCTA: Fill
		jmp  0a32h ; FCTB: Breakpoint
		jmp  0c66h ; FCTC: Copy
		jmp  0c45h ; FCTD: Delete
		jmp  0a69h ; FCTE: Single Step
BYTMOD:	jmp  0a2ah ; Einsprung fuer Byte-Mode ab 2800h
;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                   ;
;      HEXMO  MOPPEL-HEX-Monitor  V 5.6             ;
;                                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;   Stand:   Di,17.02.87;01:45:21h
;
;   Druck:
;
	org    00d0h
;ofs    00d0h
;
;
;      Monitor fuer HEX-Tastatur und Siebensegment-
;      Anzeige; unterscheidet drei Ebenen, die von
;      den Mode-FLAGs (MODFLG) abhaengig sind:
;
;      1. MEM-Mode (RAM-Zellen inspiz. u. modifiz.)
;         (Mode-Anzeige im Digit 3 ="1")
;
;      2. REG-Mode (Register inspizieren u. modif.)
;         (Mode-Anzeige im Digit 3 ="2")
;
;      3. FCT-Mode (Funktionen vorber. u. aufrufen)
;         (Mode-Anzeige im Digit 3 ="3")
;
;      In der LED-Zeile erscheint im Normalbetrieb
;      der Inhalt des Datenfeldes binaer; im Ein-
;      zelschritt-Betrieb wird hier der Inhalt
;      des CPU-FLAG-Registers angezeigt.
;
;      Aufbau der Mode-FLAGs (MODFLG):
;
;      Bit  7  6  5  4  3  2  1  0
;           :  x  x  x  :  :  :  :
;           :           :  :  :  :..Daten-Mode
;           :           :  :  :.....Adre~-Mode
;           :           :  :.....Register-Mode
;           :           :....Funktions-Eingabe
;           :............Einzelschritt-Betrieb
;
;
reset: lxi  sp,stack  ;Kaltstart
       call save_     ;Regs retten
       call init
       lxi  h,keybyt  ;KEYs true oder invers?
       mvi  m,0ffh
       in   comand    ;Ge-Diode abfragen
       cpi  7fh       ;HEX (Ge-Diode) true
       jz   mo
       inr  m         ;KEYBYT=0
       cpi  80h       ;HEX (Ge-Diode) invers
       jz   mo
       cpi  0h        ;ASCII invers
       jz   romext
       dcr  m         ;KEYBYT=FF
       jmp  romext
       call update
warm_: xra  a
       out  fdctrl    ;Hilfsfsbank 0
       lxi  sp,stack
mo:    call hexkey    ;HEX-Tastatur abfragen
       call ssgdsp    ;SSG-Anzeige aktivieren
       call cmdkey    ;Befehlstasten abfragen
       jmp  mo        ;Endlosschleife
;
;      INIT      nimmt die Grundeinstellungen vor
;
init:  lxi  h,usrstk  ;Anwender-Stack-Pointer
       mov  a,l       ;sortieren fuer REGBUF
       mov  l,h       ;(H&L vertauschen)
       mov  h,a
       shld regs      ;User-Stack definieren
       mvi  a,0h      ;REG A l|schen
       sta  modflg    ;alle Monitor-FLAGs l|schen
       sta  ioflg     ;Paralleldruck/Shift Lock
       call adrfix    ;Adr=RAMBEG
       call dat       ;Daten-Mode
       mvi  a,0h
       sta  regnr     ;"REG A" voreinstellen
       out  stpff     ;Single-Step-Flipflop
       out  fdctrl    ;Adre~latch l|schen
       out  ledpor    ;LEDs dunkel
       mvi  a,02h
       out  prmctl    ;Read Enable
       mvi  a,0ch     ;Thermodrucker-Motor
       out  prtmot    ;ausschalten
       mvi  a,20h     ;PRT-Schnittstelle
       out  sermod    ;(PRTOT auf LOW)
       mvi  a,82h     ;Steuerwort fuer 8255
       out  parctl    ;A=Out; B=In; C=Out
       mvi  a,0f0h    ;Prommer-LEDs
       out  prmled    ;ausschalten
       mvi  a,0f4h    ;Ruhepegel
       out  ezuctl    ;fuer EZU
       mvi  a,rstoff  ;RSTs sperren
       sim
       nop            ;wg. NSC800-JR-Befehl
       in   datreg    ;Reset FDC DRQ line
       out  belout    ;Schnarre aktivieren
       sta  0h        ;Res.
       ret
;
;      SAVE      rettet saemtliche Register
;
save_: sta  rega      ;REG A retten
       mov  a,l       ;H&L vertauschen
       mov  l,h
       mov  h,a
       shld regh      ;H&L retten
       push psw       ;FLAGs in den Stack
       pop  h         ;und von dort nach REG L
       mov  a,l
       sta  regf      ;FLAGs retten
       lxi  h,regb    ;Pointer auf REG-Buffer
       mov  m,b       ;REG B retten
       inx  h
       mov  m,c       ;REG C retten
       inx  h
       mov  m,d       ;REG D retten
       inx  h
       mov  m,e       ;REG E retten
       lxi  h,0h
       dad  sp        ;Stack-Pointer nach H&L
       inx  h         ;SP um Zwei erh|hen
       inx  h         ;(wegen CALL-Einsprung)
       mov  a,l       ;sortieren
       mov  l,h       ;(H&L vertauschen)
       mov  h,a
       shld regs      ;SP retten
       ret
;
;      HEXKEY    fragt die HEX-Tasten ab und schiebt
;                Eingaben ins Daten- oder Adre~feld
;                nach
;
hexkey:	call hhkey_    ;HEX-Tastenzeilen einlesen
		rnc            ;keine Taste: RET
		call shift     ;Eingabe nachschieben
		call convt     ;HEX/SSG-Umsetzung
		call reles_    ;Loslassen abwarten
		ret
;
;      	HHKEY     fragt beide HEX-Zeilen ab (ein-
;      	          schlie~lich Entprellen)
;      	no key:   ZERO-FLAG on (Z=1)
;      	key:      CARRY-FLAG on (CY=1) und
;      	          Key-Nr. in REG A und REG C
;
hhkey_:	call hkey      ;erste Abfrage
		rnc            ;keine HEX-Taste gedrueckt
		mov  b,a       ;Input in REG B retten
		call delyp     ;Prellzeit abwarten
		call hkey      ;Abfrage wiederholen
inpcmp:	cmp  b         ;beide Abfragen vergleichen
		jz   setcy     ;gleich: CY on
		stc            ;ungleich: CY off
		cmc
		ret
setcy: 	stc
		ret
;
;      	HKEY      wie HHKEY, aber ohne Entprellen
;
;
hkey:  	in   hexlo     ;Tasten 0...7 abfragen
		call keyin     ;Input-Sub
		rc             ;Taste aktiv: RET
		in   hexhi     ;Tasten 8...F abfragen
		mvi  c,8h      ;Anfangswert fuer 2.Zeile
		call keyin2    ;Input-Sub
		ret
;
;      	KEYIN     ermittelt (bei aktivierter Taste)
;      	          die Bit-Nr. innerhalb der Zeile
;      	no key:   ZERO-FLAG on (Z=1)
;      	key:      CARRY-FLAG on (CY=1) und
;      	          KEY-Nr. in REG A und REG C
;
keyin: 	mvi  c,0h      ;Anfangswert fuer 1.Zeile
keyin2:	lxi  h,keybyt  ;KEYs true./.invers?
		xra  m         ;invert.und/oder FLAGs setzen
		rz             ;kein Input: RET
;
keylop:	rrc            ;HIGH-Bit ins CY-Bit
		jc   keyend    ;Schieben beendet
		inr  c         ;Bit-Zaehler erh|hen
		jmp  keylop    ;weiter schieben
keyend:	mov  a,c       ;Ergebnis in REG A uebergeben
		ret
;
;      	SHIFT     schiebt HEX-Eingaben ins Daten-
;      	          oder Adre~feld nach (abh. von MODFL
;
shift: 	lxi  h,data    ;Pointer auf Datenfeld
		lda  modflg    ;Mode-FLAGs holen
		rrc            ;FLAG ins Carry-Bit
		jc   sftdat    ;Daten-Mode: JMP
		rrc
		jc   sftadr    ;Adre~-Mode
		ret            ;2 LSB auf LOW: RET
sftadr:	rrc            ;Reg-Flag testen
		jc   sftreg    ;Register-Mode: JMP
		inx  h         ;Pointer auf Adre~feld
		call nibadd    ;Nibble nachschieben
		mov  a,m       ;ADRLO erneut holen
		mov  m,c       ;aufdatiertes Byte nach ADRLO
		ani  0f0h      ;4 LSB l|schen
		rrc
		rrc
		rrc
		rrc
		mov  c,a       ;fuer NIBADD ablegen
		inx  h         ;Pointer auf ADRHI
sftdat:	call nibadd    ;Nibble nachschieben
		mov  m,c       ;modif.Byte ins RAM
		lda  modflg
		mov  b,a       ;MODFLG retten
		ani  8h        ;FCT-FLAG maskieren
		rz             ;kein FCT-Mode: RET
		mov  a,b       ;MODFLG holen
		ani  0feh      ;DAT-FLAG l|schen
		ori  2h        ;ADR-FLAG setzen
		sta  modflg
		ret
sftreg:	mov  a,c
		adi  6h        ;verschieben: A = 0
		ani  7h        ;MSB unterdruecken
		sta  regnr     ;hier: 0...7
		ret
;
;
;      NIBADD    schiebt das eingegebene Nibble in
;                das von H&L adressierte Byte nach
;                (Ergebnis in REG C)
;
nibadd:	mov  a,m       ;altes Byte holen
		ani  0fh       ;obere Haelfte l|schen
		rlc
		rlc
		rlc
		rlc
		ora  c         ;neues Nibble anhaengen
		mov  c,a       ;in REG C retten
		ret
;
;      READOT    aktualisiert das Datenfeld (ent-
;                sprechend Adre~feld bzw. Reg-Nr.)
;
readot:	lda  modflg    ;Mode-FLAGs holen
		mov  b,a       ;und retten
		ani  8h        ;FCT-FLAG maskieren
		rnz            ;FCT-Mode: RET
		mov  a,b       ;MODFLG zurueck holen
		ani  4h        ;REG-FLAG maskieren
		jnz  readrg    ;REG-Mode: JMP
		lhld adres     ;Adre~feld auslesen
readat:	mov  a,m       ;adressierte Daten holen
		sta  data      ;und ins Datenfeld bringen
		ret
readrg:	lxi  h,rega    ;Anfang REGBUF
		lda  regnr     ;Reg-Nr. holen
		mov  e,a
		mvi  d,0h      ;Versatz erzeugen
		dad  d         ;H&L=Adr. REGBUF
		jmp  readat
;
;      WRITIN    schreibt die Daten aus dem Daten-
;                feld ins RAM (MEM bzw. REGBUF)
;
writin:	lda  modflg    ;Mode-FLAGs holen
		ani  4h        ;REG-FLAG maskieren
		jnz  writrg    ;REG-Mode: JMP
		lhld adres     ;Adre~feld auslesen
writda:	lda  data      ;Daten holen
		mov  m,a       ;und ueberschreiben
		ret
writrg:	lxi  h,rega    ;Anfang REGBUF
		lda  regnr     ;Reg-Nr. holen
		mov  e,a
		mvi  d,0h      ;Versatz erzeugen
		dad  d         ;H&L=Zieladresse
		jmp  writda
;
;      CONVT     setzt die HEX-Daten aus dem
;                Daten- und Adre~feld in den SSG-
;                Code um und ueberschreibt diesen
;                in den Display-Buffer DSPBUF
;
convt: 	lda  data
		mov  b,a       ;Daten-Byte retten
		lda  modflg
		ani  80h       ;STP-FLAG maskieren
		jz   cnvled    ;kein Single-Step: JMP
		lda  regf      ;FLAG-Register auslesen
		mov  b,a
cnvled:	mov  a,b       ;LED-Byte holen
		sta  dspbuf    ;LED-Zeile
		lxi  d,dig1    ;Anzeige-Digit 1
		lda  data      ;Datenfeld (wieder) nach REG A
		call cnvby_    ;SSG-Umsetzung von (DATA)
		lxi  d,dig4    ;Anzeige-Digit 4
		lda  modflg    ;Mode-FLAGs holen
		ani  4h        ;REG-Mode?
		jnz  cnvreg    ;ja: JMP
cnvadr:	lda  adres
		call cnvby_    ;SSG-Umsetzung von (ADRLO)
		lda  adrhi
		call cnvby_    ;SSG-Umsetzung von (ADRHI)
		jmp  dpset
cnvreg:	lda  regnr     ;REG-Nr.
		lxi  h,regtab  ;REG-SSG-Codes
		call cnvsg2    ;SSG-Umsetzung von (REGNR)
		xra  a
		stax d         ;DIG5 l|schen
		inx  d
		stax d         ;DIG6 l|schen
		inx  d
		stax d         ;DIG7 l|schen
;
;      DPSET     Dezimalpunkt ins Daten- oder
;                Adre~feld einblenden
;
dpset: 	lxi  d,dig1    ;Pointer auf Digit 1
		lda  modflg
		ani  1h        ;DAT-FLAG maskieren
		jnz  dpcte     ;Daten-Mode: JMP
		inx  d
		inx  d
		inx  d         ;Pointer auf Digit 4
dpcte: 	ldax d         ;SSG-Code holen
		ori  80h       ;MSB setzen (=Dez.Punkt)
		stax d         ;SSG-Code wieder ablegen
		ret
;
;      CNVBYT    setzt (REG A) in den SSG-Code um;
;                ueberschreibt ihn in 2 RAM-Zellen
;                (ab (D&E)); (D&E)=(D&E)+2
;
cnvby_:	mov  b,a       ;Byte retten
		call cnvssg    ;SSG-Code holen
		mov  a,b       ;Byte zurueckholen
		rrc
		rrc
		rrc
		rrc
		call cnvssg    ;SSG-Code holen
		ret
;
;     CNVSSG    setzt REG A, unteres Nibble in
;                den SSG-Code um u.schreibt ihn
;                in die von D&E adr. RAM-Zelle;
;                erh|ht (D&E) um Eins
;
cnvssg:	lxi  h,ssgtab  ;Anfang Code-Tabelle
cnvsg2:	push d         ;Buffer-Pointer retten
		mvi  d,0h      ;Versatz bilden
		ani  0fh       ;obere 4 Bits l|schen
		mov  e,a
		dad  d         ;(H&L)=Code-Adresse
		pop  d         ;Buffer-Pointer zurueckholen
		mov  a,m       ;SSG-Code holen
		stax d         ;und in DSP-Buffer schreiben
		inx  d         ;Pointer erh|hen
		ret
;
;      SSGTAB    Tabelle der Siebensegment-Codes von
;                0...9, A...F und H,L,S,P
;
ssgtab:	db   3fh  ; 0
		db   06h  ; 1
		db   5bh  ; 2
		db   4fh  ; 3
		db   66h  ; 4
		db   6dh  ; 5
		db   7dh  ; 6
		db   07h  ; 7
		db   7fh  ; 8
		db   6fh  ; 9
regtab:	db   77h  ; A
		db   7ch  ; b
		db   39h  ; C
		db   5eh  ; d
		db   79h  ; E
		db   71h  ; F
		db   76h  ; H
		db   38h  ; L
		db   6dh  ; S
		db   73h  ; P
;
;      SSGDSP    ueberschreibt den SSG-Code aus dem
;                Display-Buffer in die Anzeige; hat
;                feste Laufzeit von 10 ms
;
ssgdsp:	lxi  d,dig7    ;oberste RAM-Zelle im DSPBUF
		inx  d
		lxi  h,hibuf   ;Output-Routine im RAM
		mvi  m,0d3h    ;HEX-Code "OUT"
		inx  h
		inx  h
		mvi  m,0c9h    ;HEX-Code "RET"
		dcx  h
		mvi  c,outpor  ;OUT-Portadresse +1
ssglop:	dcx  d         ;Adresse im DSPBUF
		mov  a,c       ;alte OUT-Adresse
		sui  10h       ;Portadresse erniedrigen
		ora  a         ;FLAGs setzen
		mov  c,a       ;OUT-Adresse retten
		mov  m,a       ;OUT-Adresse in OUT-Routine
		ldax d         ;SSG-Code aus DSPBUF holen
		call hibuf     ;OUT-Hilfsroutine
		call dely1_    ;1-kHz-MUX-Frequenz
		jnz  ssglop    ;8 Einzelausgaben abwarten
		call dely2     ;auf 10 ms verlaengern
		ret
;
;      CMDKEY    fragt die Befehlstasten ab und
;                verzweigt nach Sprungtabelle
;
cmdkey:	call cckey     ;CMD-Keys abfragen
		rnc            ;keine Taste: RET
		lxi  h,jmptab_ ;CMD-Sprungverteiler
		mov  b,a       ;Input retten
		lda  modflg
		ani  8h        ;FCT-FLAG maskieren
		jz   branch    ;CMD-Mode: JMP
		mov  a,b       ;FCT-Mode; Input holen
		cpi  nxtnr     ;NXT? (naechster Parameter)
		jz   branch    ;ja: JMP
		cpi  adrnr     ;ADR? (zurueck zum Adre~-Mode)
		jz   branch    ;ja: JMP
		cpi  fctnr     ;FCT? (zurueck zum FCT-Feld)
		jz   branch    ;ja: JMP
		cpi  runr      ;ist es RUN?
		rnz            ;nein: dann RET
;
brnch: 	lda  modflg    ;LEDASC-Sprungziel
		ani  0f7h      ;FCT-FLAG l|schen
		sta  modflg
		lxi  h,fctab   ;FCT-Sprungverteiler
		lda  data      ;"Fx" holen
		ani  0fh       ;4 MSB l|schen
		mov  b,a       ;FCT-Nr. 0...F
branch:	mov  a,b       ;Input holen
		rlc            ;Input mal 2
		add  b         ;Input mal 3
		mov  e,a
		mvi  d,0h      ;Versatz in D&E
		dad  d         ;(H&L)=Tabellen-Adresse
		lxi  d,retur   ;Sub-Ruecksprungadresse
		push d         ;im Stack ablegen
		pchl           ;Verzweigung ueber Tabelle
;
retur: 	call reles_    ;CMD-Loslassen abwarten
		ret            ;zurueck zu MONI
;
;      CCKEY     fragt die CMD-Zeile ab (ein-
;                schlie~lich Entprellen) und
;                unterdrueckt die Dioden-Nr.
;      no key:   ZERO-FLAG on (Z=1)
;      key:      CARRY-FLAG on (CY=1) und
;                Key-Nr. in REG A und REG C
;
cckey_:	call ckey      ;erste Abfrage
		rnc            ;keine CMD-Taste gedrueckt
		mov  b,a       ;Input in REG B retten
		call delyp     ;Prellzeit abwarten
		call ckey      ;Abfrage wiederholen
		jmp  inpcmp    ;Abfragen vergl.(in HHKEY)
;
;      CKEY      wie CCKEY, aber ohne Entprellen
;
ckey:  	in   comand    ;CMD-Tasten abfragen
		call keyin     ;Input-Sub
		rnc            ;keine Taste aktiv: RET
		cpi  diodnr    ;nur Ge-Diode drin?
		jnz  setcy     ;nein: CY on
		stc            ;ja: CY off
		cmc
		ret
;
;      Sprungverteiler fuer CMD-Verarbeitung
;
jmptab_:	jmp  NXT
		jmp  BST
		jmp  FCT
		jmp  REG
		jmp  ADR
		jmp  DAT
		jmp  RUN
;
;      Mode-Keys DAT,ADR/FCTF,REG,FCT (MODFLGs)
;
datse_:	shld adres     ;H&L ins Adre~feld
dat:   	lda  modflg
		ani  0fdh      ;ADR-FLAG l|schen
		ori  1h        ;DAT-FLAG setzen
		jmp  modcte
adrfix:	lxi  h,rambeg  ;RAM-Anfangsadresse
adrse-:	shld adres     ;ins Adre~feld laden
adr:   	mvi  a,8h      ;ein Strich
		sta  dig3      ;(ADR-Mode-Kennzeichen)
		lda  modflg
		ani  0f2h      ;DAT+REG+FCT-FLAGs l|schen
		ori  2h        ;ADR-FLAG setzen
		jmp  modcte
reg:   	mvi  a,48h     ;zwei Striche
		sta  dig3      ;(REG-Mode-Kennzeichen)
		lda  modflg
		ani  0feh      ;DAT-FLAG l|schen
		ori  6h        ;ADR+REG-FLAGs setzen
		jmp  modcte
fct:   	mvi  a,49h     ;drei Striche
		sta  dig3      ;(FCT-Mode-Kennzeichen)
		mvi  a,7h      ;Max. fuer 4 Parameter
		sta  maxbuf
		xra  a
		sta  nxtbuf    ;NXTBUF l|schen
		mvi  a,0ffh
		sta  data      ;"FF" ins Datenfeld
		lxi  h,0h
		shld adres     ;"0000" ins Adre~feld
		lda  modflg
		ani  0f9h      ;ADR+REG-FLAGs l|schen
		ori  9h        ;FCT+DAT-FLAGs setzen
modcte:	sta  modflg
update:	call readot    ;Daten ins Datenfeld
		call convt     ;SSG-Umsetzung
		ret
;
;      NXT       erh|ht Adr/Reg und schreibt
;                Daten(feld) ins RAM
nxt:   	lda  modflg
		ani  8h        ;FCT-FLAG
		jnz  nxtcte    ;FCT-Mode: JMP
		call writin    ;Daten einschreiben
		jnz  nxtreg    ;REG-Mode: JMP
		inx  h         ;Adre~feld hochzaehlen
		shld adres     ;und ablegen
		jmp  update    ;Anzeige aktualisieren
nxtreg:	lda  regnr
		inr  a         ;Reg-Nr. hochzaehlen
		daa            ;Dezimalkorrektur
nxtsto:	ani  0fh       ;]bertrag unterdruecken
		sta  regnr     ;neue Reg-Nr. ablegen
		jmp  update    ;Anzeige aktualisieren
nxtcte:	lxi  h,maxbuf  ;Maximalwert f.Eingaben
		lda  nxtbuf    ;Eingabe-Zaehler
		cmp  m         ;Maximalwert erreicht?
		rnc            ;ja: RET
		lhld adres     ;letzter Parameter
		xchg           ;nach D&E retten
		lxi  h,srcbeg  ;Anf.Parameter-Buffer
		mov  c,a       ;Versatz erzeugen
		mvi  b,0h
		dad  b         ;SRCBEG + NXTBUF
		mov  m,e       ;Parameter in Buffer
		inx  h
		mov  m,d
		lxi  h,0h
		shld adres     ;Adre~feld wieder l|schen
		lxi  h,nxtbuf  ;Eingabe-Zaehler +2
		inr  m
		inr  m
		jmp  update    ;Adr=0000 anzeigen
;
;      BST       schaltet Adr/Reg zurueck
;
bst:   	call readot    ;Pointer ermitteln
		jnz  bstreg    ;REG-Mode: JMP
		dcx  h         ;Adre~feld erniedrigen
		shld adres     ;und ablegen
		jmp  update    ;Anzeige aufdatieren
bstreg:	lda  regnr     ;Reg-Nr. holen
		dcr  a         ;und erniedrigen
;
		jp   nxtsto    ;kein neg.]bertrag: JMP
		mvi  a,9h      ;auf Anfangswert setzen
		jmp  nxtsto    ;neue Reg-Nr. ablegen
;
;      RUN       Programmstart bei der im Adre~feld
;                stehenden Adresse; laedt vorher alle
;                Register (einschlie~lich FLAGs)
;
run:   	xra  a         ;REG A l|schen
		sta  dspbuf    ;LEDs dunkel schalten
		call reles_    ;auf Loslassen warten
resto_:	lda  modflg
		ani  80h       ;STP-Mode aktiv?
		cnz  rst1      ;ja: Breakpt.setzen
		lda  regf      ;FLAGs holen
		mov  c,a
		push b         ;und in den Stack
		pop  psw       ;von dort ins CPU-FLAG-Reg.
		lhld regs      ;Anwender-Stack-Pointer holen
		mov  a,l       ;und sortieren
		mov  l,h       ;(H&L vertauschen)
		mov  h,a
		sphl           ;in CPU-Stack-Pointer
		lxi  h,regb    ;Pointer auf REG-Buffer
		mov  b,m       ;REG B laden
		inx  h
		mov  c,m       ;REG C laden
		inx  h
		mov  d,m       ;REG D laden
		inx  h
		mov  e,m       ;REG E laden
		lhld adres     ;Startadresse holen
		push h         ;und im Stack ablegen
		lhld regh      ;Daten fuer H&L holen
		mov  a,l       ;und sortieren
		mov  l,h       ;(H&L vertauschen)
		mov  h,a       ;H&L sind geladen
		lda  rega      ;REG A laden
		ret            ;Programmstart
;
;      RELES     wartet auf Loslassen einer aktiven
;                Taste und aktiviert waehrend dieser
;                Zeit die SSG-Anzeige
;
reles_:	call rls       ;Abfrage-Unterprogramm
		jc   reldsp    ;Taste betaetigt: JMP
		call delyp     ;Prellzeit abwarten
		call rls       ;erneute Abfrage
		rnc            ;keine Taste: RET
reldsp:	call ssgdsp    ;SSG-Anzeige aktivieren
		jmp  reles_    ;neuer Abfrage-Zyklus
;
;      RLS       fragt nacheinander HEX- und CMD-
;      	          Keys ab (kein Entprellen)
;      	no key:   ZERO-FLAG on (Z=1)
;      	key:      CARRY-FLAG on (CY=1)
;
rls:   	call hkey      ;HEX-Keys abfragen
		rc             ;HEX-Taste aktiv: RET
		call ckey      ;CMD-Keys abfragen
		ret
;
;      PRESS     springt bei Tastendruck zurueck
;                zum Warmstart; sonst Return
;
press_:	call dly10_    ;Anzeige aktivieren
		call hhkey_
		jc   preles    ;HEX-KEY aktiv: JMP
		call cckey_
		rnc            ;keine Taste: RET
preles:	call reles_    ;Loslassen abwarten
		call adrfix    ;Adr=RAM-Beginn
		jmp  warm
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Laufzeit-Unterprogramme
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
onese_:	push psw       ;  1 s
		mvi  a,0ah     ; 10 d
onelop:	call dly10_
		dcr  a
		jnz  onelop
		pop  psw
		ret
;
dly10_:	push psw       ; 100 ms
		mvi  a,0ah
delylp:	push psw
		push h
		push d
		push b
		call ssgdsp
		pop  b
		pop  d
		pop  h
		pop  psw
		dcr  a
		jnz  delylp
		pop  psw
		ret
;
delyp: 	push psw       ;KEY-Prellzeit
		lda  timprl
dlop:  	call dely1_
		dcr  a
		jnz  dlop
		pop  psw
		ret
;
dely2: 	call dely1_    ; 2 ms
dely1_:	push psw       ; 1 ms
		lda  fquarz    ;4/6/8 MHz!
		add  a
		add  a
		inr  a         ;adjust to 99%
		add  a
		add  a
		add  a         ;Zeitfaktor ;32
delop: 	dcr  a
		jnz  delop
		pop  psw
		ret
;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                   ;
;      EZU/DSP  Echtzeit-Uhr und SSG-Anzeige        ;
;                                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;   Stand:   Di,17.02.87;08:56:01h
;
;   Druck:
;
	org    04c0h
;
;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Stellen der Echtzeit-Uhr, danach auslesen
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
fctwur:	lhld adres     ;letzte Eing.(Mon/Jahr)
		xchg           ;nach D&E
		lhld srcend    ;vorl. Eing.(Wotag,Kaltag)
		mov  c,l       ;nach B&C
		mov  b,h
		lhld srcbeg    ;erste Eing.(Std,Min)
		call writu_    ;Zeit einschreiben
		jmp  fctrur    ;und anzeigen
;
;      WRITUR    Stellt die 5832-Echtzeit-Uhr
;
;      (H&L):    Stunden, Minuten (Sekunden=00)
;      (B&C):    Wochentag (1=Mo), Kalendertag
;      (D&E):    Monat, Jahres-Einer und -Zehner
;
WRITU_: 	PUSH    D       ;MON/JAR RETTEN
		PUSH    B       ;WTG/KTG RETTEN
		MOV     A,H
		ORI     80H     ;24-h-Format
		MOV     H,L
		MOV     L,A     ;H&L=MIN/STD
		MVI     C,8h    ;ADRESSE SEKUNDEN-EINER
		MOV     A,C     ;HOLD=H; WRITE=L
		OUT     EZUCTL
		CALL    DELY1   ;Wartezeit 1 ms (150 us)
		PUSH    H       ;MIN/STD RETTEN
		MVI     L,0h    ;SEK = 00
		CALL    LOUT    ;SEKUNDEN L\SCHEN
		POP     H       ;MIN/STD ZUR]CKHOLEN
		CALL    HLOUT   ;MIN/STD
		POP     D       ;WTG/KTG ZUR]CKHOLEN
		XCHG
		MOV     A,H
		ANI     7h
		DCR     A       ;MSM-WTG-FORMAT 0...6
		CALL    OUTPT   ;WTG
		CALL    LOUT    ;KTG
		POP     D       ;MON/JAR ZUR]CKHOLEN
		XCHG
		CALL    HLOUT   ;MON/JAR
		MVI     A,0F4H  ;HOLD=L; READ=H
		OUT     EZUCTL
		RET
;
HLOUT:  	MOV     A,H
		CALL    OUTPUT
LOUT:   	MOV     A,L
		CALL    OUTPUT
		RET
;
OUTPUT: 	PUSH    PSW
		ANI     0FH
		CALL    OUTPT
		POP     PSW
		RRC
		RRC
		RRC
		RRC
		ANI     0FH
		CALL    OUTPT
		RET
;
OUTPT:  	OUT     EZUDAT  ;DATEN AUSGEBEN
		MOV     A,C
		OUT     EZUCTL  ;ADRESSE AUSGEBEN
		ORI     2h      ;H-PULS AN WRITE
		OUT     EZUCTL
		ANI     0FDH
		OUT     EZUCTL
		MVI     A,10H   ;ADRESSE ERH\HEN
		ADD     C
		MOV     C,A
		RET
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Echtzeit-Uhr auslesen und anzeigen
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
fctrur:	call ezuds_    ;EZU auslesen und anzeigen
		jmp  fctrur
;
;      READUR    Liest die 5832-Echtzeit-Uhr aus
;                und ueberschreibt Uhrzeit und Da-
;                tum in den UHRBUF ins RAM
;
;
READU_: 	LXI     H,UHRBUF
		PUSH    H
		XCHG            ;RAM-ANFANG NACH D,E
		MVI     C,0CH   ;ANFANGSADRESSE 5832
		MVI     A,8h    ;HOLD=H; READ=L
		OUT     EZUCTL
		CALL    DELY1   ;Wartezeit 1 ms (150 us)
;
INLOP:  	MOV     A,C     ;ADRESSE HOLEN
		OUT     EZUCTL  ;UND AUSGEBEN
		PUSH    H       ;Wartezeit von 6 us
		POP     H
		IN      EZUDAT  ;DATEN EINLESEN
		STAX    D
		INX     D
		MVI     A,10H   ;ADRESSE ERH\HEN
		ADD     C
		MOV     C,A
		CPI     0DCH    ;ENDABFRAGE
		JNZ     INLOP
		POP     H
		CALL    SORT    ;F]R UHRBUF FORMATIEREN
		MVI     A,0F4H  ;HOLD=L; READ=H
		OUT     EZUCTL
		RET
;
SORT:   	PUSH    H       ;RAM-ANF.ADR. RETTEN
		LXI     B,6h    ;OFFSET WOCHENTAG
		DAD     B
		MOV     B,M     ;WOCHENTAG RETTEN
		POP     H
		PUSH    H
		XCHG            ;QUELL-POINTER: D,E
		POP     H
		PUSH    H       ;ZIEL-POINTER:  H,L
		CALL    BIT4    ;SEKUNDEN-EINER
		CALL    BIT3    ;SEKUNDEN-ZEHNER
		CALL    BIT4    ;MINUTEN-EINER
		CALL    BIT3    ;MINUTEN-ZEHNER
		CALL    BIT4    ;STUNDEN-EINER
		LDAX    D       ;STUNDEN-ZEHNER
		ANI     3h
		MOV     M,A
		INX     D
		INX     D
		INX     H
		CALL    BIT4    ;KAL'TAG-EINER
		LDAX    D       ;KAL'TAG ZEHNER
		ANI     3h
		MOV     M,A
		INX     D
		INX     H
		INX     H
		CALL    BIT4    ;MONATS-EINER
		LDAX    D       ;MONATS-ZEHNER
		ANI     1h
		MOV     M,A
		INX     D
		INX     H
		CALL    BIT4    ;JAHRES-EINER
		CALL    BIT4    ;JAHRES-ZEHNER
		DCX     D
		MVI     C,4h
SHIFT_:  	LDAX    D
		MOV     M,A
		DCX     D
		DCX     H
		DCR     C
		JNZ     SHIFT_
		MOV     A,B     ;WOCHENTAG ZUR]CKHOLEN
		ANI     7h
		INR     A       ;NORMALFORMAT WTG
		MVI     M,0h    ;WTG-DUMMY
		DCX     H
		MOV     M,A
		POP     H
		RET
;
BIT3:   	LDAX    D       ;BYTE HOLEN
		ANI     7h      ;3 LSB MASKIEREN
		JMP     BITCTE
BIT4:   	LDAX    D       ;BYTE HOLEN
		ANI     0FH     ;4 LSB MASKIEREN
BITCTE: 	MOV     M,A
		INX     D       ;POINTER ERH\HEN
		INX     H
		RET
;
;
;      EZUDSP    Uhrzeit in die Siebensegment-
;                Anzeige ueberschreiben
;
ezuds_:	call readu_    ;Uhrzeit auslesen
		lxi  d,dig1
		call ezusub    ;Sekunden umsetzen
		inx  d         ;DIG3 ueberspringen
		call ezusub    ;Minuten umsetzen
		call ezusub    ;Stunden umsetzen
		lda  dig6
		ori  80h       ;DP einblenden
		sta  dig6
		call dly100    ;SSG-Buffer anzeigen
		xra  a         ;REG A l|schen
		sta  dspbuf    ;LEDs dunkel
		ret
;
ezusub:	mov  b,m       ;erstes BCD-Digit
		inx  h
		mov  a,m       ;zweites BCD-Digit
		inx  h
		rlc
		rlc
		rlc
		rlc
		ora  b         ;beide verknuepfen
		push h
		call cnvbyt    ;und in SSG-Code umsetzen
		pop  h
		ret
;
;
;      TIME      Liest die 5832-Echtzeit-Uhr in
;                RAM-Hilfsbuffer aus (in den Uhr-
;                zeit-Buffer UHRBUF) und setzt
;                die Uhrzeit- und Datums-Informa-
;                tion in den ASCII-Code um (ab
;                2FEAh, durch "00" abgeschlossen)
;                H&L steht anschl.auf Stringanfang
;
TIME_:  	CALL    READU_  ;EZU AUSLESEN
		XCHG            ;D&E: UHRBUF
		DCX     D
		LXI     H,2FFFH ;ENDE ASCII-BUF
		MVI     M,0h    ;00 = STRING-ENDE
		DCX     H
		MVI     M,"h"
		CALL    ASCII2  ;SEKUNDEN
		DCX     H
		MVI     M,3ah   ;":"
		CALL    ASCII2  ;MINUTEN
		DCX     H
		MVI     M,3ah   ;":"
		CALL    ASCII2  ;STUNDEN
		DCX     H
		MVI     M,";"
		LXI     D,MONZ
		CALL    ASCII2  ;JAHR
		DCX     H
		MVI     M,"."
		LXI     D,WTGU
		CALL    ASCII2  ;MONAT
		DCX     H
		MVI     M,"."
		LXI     D,STDZ
		CALL    ASCII2  ;KALENDERTAG
		DCX     H
		MVI     M,2ch   ;","
		DCX     H
		PUSH    H
		LXI     H,WTGTAB
		LDA     WTGL    ;WTG-Nr.
		ANI     7h
		RLC
		MOV     E,A
		MVI     D,0h
		DAD     D
		MOV     E,M     ;WTG-SYMBOL (LOWER)
		INX     H
		MOV     D,M     ;WTG-SYMBOL (UPPER)
		POP     H
		MOV     M,E
		DCX     H
		MOV     M,D
		RET
;
;
;
ASCII2: 	CALL    ASCII1
ASCII1: 	DCX     H
		INX     D
		LDAX    D
		adi     30h    ;ASCII-Umsetzung
		cpi     3ah    ;"9" + 1
		jnc     stroke ;gr|~er 9: JMP
		cpi     "0"
		jnc     ascte  ;gr|~er/gleich 0: JMP
stroke: 	mvi     a,"-"
ascte:  	mov     m,a    ;Nibble in ASCBUF
		ret
;
WTGTAB: 	NOP
		NOP
		DB      "o"
		DB      "M"
		DB      "i"
		DB      "D"
		DB      "i"
		DB      "M"
		DB      "o"
		DB      "D"
		DB      "r"
		DB      "F"
		DB      "a"
		DB      "S"
		DB      "o"
		DB      "S"
;
;
;      DSPACC    Akku-Inhalt im Siebensegment-
;                Code und binaer in LED-Zeile dar-
;                stellen; Warmstart bei Tastendruck
;
dspac_:	push psw
		lxi  d,dig1
		call cnvbyt    ;SSG-Umsetzung
		pop  psw
		sta  dspbuf    ;(ACC) in LED-Zeile
		lxi  h,0h
		shld dig4      ;obere Digits dunkel
		shld dig6
dsplop:	call press     ;auf Taste warten
		jmp  dsplop
;
;
;      	DSPH&L    Inhalt von H&L im Siebensegment-
;      	          Code darstellen; Warmstart bei
;      	          Tastendruck
;
dsphl_:	push h         ;H&L retten
		lxi  d,dig4    ;rechte Adre~anzeige
		mov  a,l       ;(REG L) SSG umsetzen
		call cnvbyt
		pop  h         ;H&L wiederholen
		mov  a,h       ;(REG H) SSG umsetzen
		call cnvbyt
dspcte:	lxi  h,0h      ;untere Digits dunkel
		shld dig1
		jmp  dsplop
;
;
;      DSPERR    Fehleranzeige "ErrX"
;                bei Tastendruck zurueck zum
;                Warmstart
;
dsper_:	lxi  d,dig4    ;rechte Adre~anzeige
		call cnvbyt    ;Fehler-Nummer
		lxi  h,dig7
		mvi  m,79h     ; "E"
		dcx  h
		mvi  m,50h     ; "r"
		dcx  h
		mvi  m,50h     ; "r"
		jmp  dspcte
;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                   ;
;      PERIPH  Ansteuerung von Peripheriegeraeten    ;
;                                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;   Stand:   Di,17.02.87;09:27:36h
;
;   Druck:
;
	org    06a2h
;ofs    06a2h
;
;
;      Unterprogramme fuer Peripheriegeraete:
;
;      - LOINIT initialisiert PRT-IF
;      - CSINIT initialisiert CAS-IF
;      - V24INI initialisiert V.24-Schnittstelle
;      - DF]INI initialisiert DF]-Schnittstelle
;      - SERINI init.SER-Schn.st.:H&L:Baudr.; A:Mode
;
;      - PO Cassetten-Ausgabe
;      - RI Cassetten-Eingabe
;      - LO Drucker-Ausgabe
;
;      - CPUOT SOD-Ausgabe (8085-CPU)
;      - CPUIN SID-Eingabe (8085-CPU)
;
;
adrot  equ  103ch     ;(H&L) anzeigen
updat  equ  2e56h     ;Cursor-Update-Adresse
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Schnittstellen initialisieren
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
loini_:	push d         ;Drucker-Schnittstelle
		push h
locen: 	mvi  a,1h      ;Centronics
		out  censtb
lopar: 	mvi  a,8ah     ;8255 parallel
		out  parctl
		mvi  a,1h
		out  parpoc
lotho: 	lxi  h,txtbuf  ;Thermodrucker
		shld txtpoi
		xra  a
		sta  txtnum
loser: 	lda  prtmod    ;seriell
		mov  b,a
		mvi  a,0a0h
		lhld prtbdr
		jmp  initct
;
csini_:	push d         ;Magnetband-Schnittstelle
		push h
		lda  casmod
		mov  b,a
		mvi  a,0c2h
		lhld casbdr
		jmp  initct
;
v24in_:	push d         ;V.24-Schnittstelle
		push h
		lda  v24mod
		lhld v24bdr
		jmp  v24cte
;
dfueini:	push d         ;DF]-Schnittstelle
		push h
		lda  dfuemod
		lhld dfuebdr
		jmp  v24cte
;
serini:	push d         ;V.24 m.variabl.Baudrate/Mode
		push h
v24cte:	mov  b,a
		mvi  a,48h
;
initct:	out  sermod    ;Mode-Wort an '374
		mvi  a,3h      ;RESET fuer 6850
		out  serctl
		mvi  a,36h     ;Timer-CTL: Mode 3 binaer
		out  timctl
		call baudrt    ;Teilerfaktor ermitteln
		mov  a,l
		out  timdat    ;Teilerfaktor (lower)
		mov  a,h
		out  timdat    ;Teilerfaktor (upper)
		mov  a,b
		out  serctl    ;Mode-Wort 6850
		pop  h
		pop  d
		ret
;
;      BAUDRT    ermittelt aus der Baudrate und
;                der CPU-Quarzfrequenz FQUARZ den
;                Timer-Teilerfaktor; zulaessige
;                Werte: 9600,4800,2400,1200,600,
;                500 (Dummy! i.e.300 Bd.DF]; CLK/16),
;                300 und 110 Bd; Quarz: ca.4 MHz (1)
;                ca. 6 MHz (2) oder ca. 8 MHz (3)
;
baudrt:	mov  a,h       ;Baudrate (upper) holen
		rlc
		mvi  a,0h
		adc  h
		ani  7h        ;Versatz
		rlc
		mov  e,a
		mvi  d,0h
		lda  fquarz
		lxi  h,tab3    ;hohe Frequenz
		cpi  7h
		jnc  bdend
		lxi  h,tab2    ;mittlere Frequenz
		cpi  5h
		jnc  bdend
		lxi  h,tab1    ;niedrige Frequenz
bdend: 	dad  d
		mov  e,m
		inx  h
		mov  d,m
		xchg           ;H&L: Teilerfaktor
		ret
;
tab1:  ds   0h        ; 4.194 MHz
       dw   01b5h     ; 4800 Bd
       dw   4a78h     ;  110 Bd
       dw   06d4h     ; 1200 Bd
       dw   1b4eh     ;  300 Bd
       dw   036ah     ; 2400 Bd
       dw     6dh     ;  300 Bd; CLK/16
       dw   0da7h     ;  600 Bd
       dw   00dah     ; 9600 Bd
;
tab2:  ds   0h        ; 6.144 MHz
       dw   0280h     ; 4800 Bd
       dw   6d17h     ;  110 Bd
       dw   09c4h     ; 1200 Bd
       dw   2800h     ;  300 Bd
       dw   0500h     ; 2400 Bd
       dw     9ch     ;  300 Bd; CLK/16
       dw   1400h     ;  600 Bd
       dw   0140h     ; 9600 Bd
;
tab3:  ds   0h        ; 8.000 MHz
       dw   0341h     ; 4800 Bd
       dw   8e0ch     ;  110 Bd
       dw   0d05h     ; 1200 Bd
       dw   3415h     ;  300 Bd
       dw   0683h     ; 2400 Bd
       dw    0d0h     ;  300 Bd; CLK/16
       dw   1a0bh     ;  600 Bd
       dw   01a1h     ; 9600 Bd
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Cassetten-Ausgabe
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Das Programm erzeugt ca.5 s langen Vorspann,
;      um das spaetere Einlesen definiert starten
;      zu k|nnen
;
fctcot:	lhld adres     ;letzten Parameter holen
		shld srcend    ;und ablegen
;
casot_:	lda  srcbeg    ;Abfrage wg. Abgleich
		cpi  0adh      ;ad=Adjust
		jz   adjust    ;ja: JMP
;
		call csini_    ;CAS-Ausgabe
		lxi  b,258h    ;600d ; 8,3 ms
outlop:	xra  a         ;REG A l|schen
		call casot1    ;Byte ausgeben
		dcx  b
		cmp  c
		jnz  outlop
		cmp  b
		jnz  outlop
;
		mvi  a,0d3h    ;Startzeichen
		call casot2    ;3mal ausgeben
		call casot1
;
		xra  a
		call casot1    ;"00" hinterher
;
		lhld srcend    ;Parameter holen
		inx  h
		xchg
		lhld srcbeg
output_:	mov  c,m       ;Zeichen holen
		call po_       ;und ausgeben
		inx  h
		mov  a,l       ;Endabfrage
		cmp  e
		jnz  output_    ;nicht die Endadresse: JMP
		mov  a,h
		cmp  d
		jnz  output_    ;nicht die Endadresse: JMP
;
		xra  a
		call casot2    ;4mal "00" als
		call casot2    ;Endzeichen
		ret
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Cassetten-Eingabe
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Programm im 5-s-Vorspann starten
;
fctcin:	lhld adres     ;letzten Parameter holen
		shld srcend    ;und ablegen
		call casin_
		rnz            ;o.k.: RET
		jmp  dsperr    ;Einlesefehler
;
casin_:	call csini_
		mvi  b,64h     ;100d Bytes erfassen
inlop0:	mvi  c,0ah     ;10d Vorlauf-Bytes
inlp0: 	call ri_
		cpi  0h
		jz   inl0      ;"00" erkannt
		dcr  b
		jnz  inlop0
		jmp  invert
inl0:  	dcr  c
		jnz  inlp0
		jmp  inlop2    ;auf Startmarke "D3" warten
;
invert:	mvi  a,0c3h    ;Signal invertieren (mit LSB)
		out  sermod
;
		mvi  b,64h     ;100d Bytes erfassen
inlop1:	mvi  c,0ah     ;10d Vorlauf-Bytes
inlp1: 	call ri_
		ora  a
		jz   inl1      ;"00" erkannt
		dcr  b
		jnz  inlop1
		mvi  a,50h     ;Vorspann-Fehler
		jmp  inend     ;Fehler: Z=1
inl1:  	dcr  c
		jnz  inlp1
;
inlop2:	lxi  b,200h    ;auf Anfang warten
inlop3:	call ri
		cpi  0d3h
		jz   inlop4
		dcx  b
		mov  a,b
		ora  c
		jnz  inlop3
		mvi  a,51h     ;Startzeichen-Fehler
		jmp  inend
;
inlop4:	mvi  c,2h      ;2 weitere Start-Bytes
inlop5:	call ri_
		cpi  0d3h
		mvi  a,51h     ;Startzeichen-Fehler
		jnz  inend
		dcr  c
		jnz  inlop5
;
inlop6:	call ri_       ;Separator
		ora  a
		mvi  a,52h
		jnz  inend     ;Fehler: Z=1
;
input: 	lhld srcend    ;Parameter holen
		inx  h
		xchg
		lhld srcbeg
inplop:	call ri_       ;Zeichen einlesen
		mov  m,a       ;sonst im RAM ablegen
		mvi  a,55h
		jz   inend     ;Datenverlust
		inx  h
		mov  a,l       ;Endabfrage
		cmp  e
		jnz  inplop    ;nicht die Endadresse: JMP
		mov  a,h
		cmp  d
		jnz  inplop    ;nicht die Endadresse: JMP
		xra  a
		inr  a         ;o.k.: ZERO off
		ret
inend: 	mov  b,a
		xra  a         ;Err.: ZERO on
		mov  a,b
		ret
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Cassetten-Interface abgleichen
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Testpunkt TP0 an Masse legen, Testpunkt TP1 mit
; Timer-Eingang G0 verbinden und Abgleich-Routine
; aufrufen: Eingabe "o ad", gefolgt von Return
; ("o" fuer "Cassetten-Ausgabe", "ad" fuer "Adjust").
;
;      Sollwert bei 4.000 MHz: 3750d
;                   4.194 MHz: 3690d
;                   6.000 MHz: 3125d
;                   6.144 MHz: 3080d
;                   8.000 MHz: 2500d
;
adjust:	mvi  a,31h     ;Mode 0, BCD
		out  timctl
		mvi  a,0h
		out  timdat
		mvi  a,50h     ;Anfangswert 5000d
		out  timdat
;
		mvi  a,1h
		out  sermod    ;XOR triggern
		xra  a
		out  sermod
		call dely1     ;1 ms abwarten
;
		mvi  a,1h      ;Timer lesen
		out  timctl
		in   timdat    ;lower Byte
		mov  l,a
		in   timdat    ;upper Byte
		mov  h,a
		xchg
		lhld updat     ;Cursor-Update
		push h
		xchg
		call adrot     ;Zaehlerstand anzeigen
		pop  h
		shld updat
		call onesec    ;beruhigen
		jmp  adjust
;
;
;     CASOT1/2   gibt 1 bzw. 2 Bytes ueber das
;                Magnetband-Interface aus (auf
;                der Seriellen Interface-Karte)
;
casot2:	call casot1
casot1:	push b
		mov  c,a
		push psw
		call po_       ;Zeichen ausgeben
		pop  psw
		pop  b
		ret
;
;      RI        liest ein Byte ueber das Magnet-
;                band-Interface ein (ins REG A)
;
ri_:   	push b
		mvi  c,0ch     ;max. Wartezeit: 12 ms
rilop: 	in   serctl    ;Rec.Data full?
		ani  1h        ;Statusbit maskieren
		jnz  ricte
		call dely1
		dcr  c
		jnz  rilop
		mvi  a,3h      ;Fehler: Datenverlust
		pop  b
		ret            ;Fehler: Z=1
ricte: 	in   serdat    ;Daten auslesen
		pop  b
		ret
;
;      PO        gibt ein Byte ueber das Magnet-
;                band-Interface aus (von REG C)
;
po_:  	 mov  b,c
		mvi  c,0ch     ;max. Wartezeit 12 ms
polop:	 in   serctl    ;Transm.Data empty?
		ani  2h        ;Statusbit maskieren
		jnz  pocte
		call dely1
		dcr  c
		jnz  polop
		mvi  a,4h      ;Gegenstelle nicht bereit
		ret            ;Fehler: Z=1
pocte:	 mov  a,b
		out  serdat    ;Daten uebergeben
		ret
;
;      LO        gibt ein Byte ueber die MOPPEL-
;                Schnittstellen aus; die Auswahl
;                der vier m|glichen LO-Geraete er-
;                folgt ueber die I/O-FLAGs
;
;      LO7: V.24-Schnittstelle (IOFLAG #7)
;      LO6: Centronics-Schnittstelle (IOFLAG #6)
;      LO5: 8255-Parallel-Ausgabe (IOFLAG #5)
;      LO4: Thermodrucker (IOFLAG  #4)
;
lo_:  	lda  ioflg
		ani  0f0h      ;4 MSB maskieren
		rlc
		jc   lo7
		rlc
		jc   lo6
		rlc
		jc   lo5
		rlc
		jc   lo4
		xra  a         ;FLAGs setzen
		mvi  a,5h      ;I/O-Byte undefiniert
		ret            ;Fehler: Z=1
;
;      Serielle Ausgabe ueber die Drucker-
;      Schnittstelle
;
lo7:  	push b
lolop:	in   serctl
		ani  2h        ;Transmission Data
		jz   lolop     ;Reg.noch nicht leer
locts:	in   serctl
		ani  8h        ;Clear to send
		jnz  locts     ;Dev.noch busy
		lxi  b,0h
		push b
		pop  psw       ;FLAGs l|schen
		pop  b
		mov  a,c
		out  serdat    ;Zeichen uebergeben
		ret
;
;     	Parallel-Ausgabe ueber Centronics-
;     	Schnittstelle
;
lo6:  	in   cendat
		ani  40h       ;BUSY
		jz   lo6       ;Dev.not ready
		mov  a,c
		out  cendat    ;Daten ausgeben
		out  censtb    ;STROBE-Puls
		ret
;
;     	Parallel-Ausgabe ueber 8255
;
;     	Daten: Port A
;     	BUSY:  Port C, Bit 7 (In)
;     	STROBE:Port C, Bit 0 (Out)
;
lo5:  	in   parpoc
		ani  80h       ;BUSY
		jnz  lo5       ;Dev.not ready
		mov  a,c
		out  parpoa    ;Daten ausgeben
		xra  a
		out  parpoc    ;STROBE-Puls
		call lo5end    ;Wartezeit
		inr  a
		out  parpoc
lo5end:	ret
;
;      Ausgabe ueber den Thermodrucker als
;      LO-Ausgabegeraet (Hilfsbank 1; 40h an FDCTRL)
;
prthex	equ	2000h          ;HEX-Dump
prtasc	equ	2003h          ;ASCII-Ausdruck
prtlin	equ	2014h          ;Zeile (ASCII) drucken
;     	equ
txtpoi	equ	2e00h          ;Buffer-Pointer
txtnum	equ	2e02h          ;Zeichenzaehler
txtbuf	equ	2e03h          ;Text-Buffer
;
lo4:   	mov  a,c
		cpi  08h       ;BS?
		jz   del
		cpi  0ch       ;FF?
		jz   ff
		cpi  0dh       ;CR?
		jz   cr
		cpi  7fh       ;DEL?
		jz   del
;
		cpi  20h       ;Code im zulaessigen Bereich?
		jc   lo4ret    ;nein: JMP und uebergehen
		cpi  0a0h
		jc   lo4cte
lo4ret:	nop            ;./. XRA A wg. Fehlermeldung
		mvi  a,9h      ;unzulaessiger Code
		ret            ;evtl. Z=1 uebergeben
;
lo4cte:	lhld txtpoi
		mov  m,c       ;Zeichen in Text-Buffer
		inx  h         ;Buffer-Pointer erh|hen
		shld txtpoi
;
		lda  txtnum
		inr  a         ;Zeichenzaehler erh|hen
		sta  txtnum
		cpi  14h       ;Buffer voll?
		rnz            ;nein: RET
;
cr:    	xra  a         ;Zeichenzaehler l|schen
		sta  txtnum
		lxi  h,txtbuf  ;Pointer auf Textanfang
		shld txtpoi
		mvi  a,40h
		out  fdctrl
		call prtlin    ;Zeile ausdrucken
		xra  a
		out  fdctrl
;
ff:    	mvi  a,14h     ;Buffer l|schen
		lxi  h,txtbuf
fflop: 	mvi  m,20h
		inx  h
		dcr  a
		jnz  fflop
		inr  a         ;FLAGs l|schen
		ret
;
del:   	lda  txtnum    ;Zeichen l|schen
		dcr  a
		rm             ;Buffer war leer: RET
		sta  txtnum
		lhld txtpoi
		dcx  h
		mvi  m,20h     ;Blank ueberschreiben
		shld txtpoi
		ret
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Serielle Schnittstellen der CPU ansteuern
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Datenuebertragung zwischen SOD des einen und
;      SID des anderen Computers; dazu folgende
;      Verbindungen herstellen:
;      SID, Geraet A  an  SOD, Geraet B
;      SOD, Geraet A  an  SID, Geraet B
;      GND, Geraet A  an  GND, Geraet B (Masse)
;
		jmp  cpuot     ;Sprungziel
fctsod:	lhld adres     ;SOD-Ausgabe
		inx  h
		xchg           ;D&E: Endadresse
		lhld srcbeg    ;H&L: Startadresse
otlop: 	mov  c,m       ;Zeichen holen
		call cpuot     ;und ausgeben
		inx  h
		mov  a,l       ;Endabfrage
		cmp  e
		jnz  otlop     ;nicht die Endadresse: JMP
		mov  a,h
		cmp  d
		jnz  otlop     ;nicht die Endadresse: JMP
		ret
;
		jmp  cpuin     ;Sprungziel
fctsid:	lhld adres     ;SID-Eingabe
		inx  h
		xchg           ;D&E: Endadresse
		lhld srcbeg    ;H&L: Startadresse
		mvi  a,0c0h    ;SOD auf HIGH
		sim            ;(=Ready)
inlops: 	call cpuin     ;Zeichen einlesen
		mov  m,a       ;und im RAM ablegen
		inx  h
		mov  a,l       ;Endabfrage
		cmp  e
		jnz  inlops     ;nicht die Endadresse: JMP
		mov  a,h
		cmp  d
		jnz  inlops     ;nicht die Endadresse: JMP
		ret
;
;
;      CPUOT     Ausgabe von (REG C) ueber die CPU-
;                Schnittstelle SOD
;
cpuot: 	rim            ;Ready abfragen
		ana  a
		jp   cpuot     ;SID auf LOW: JMP
		mov  a,c       ;Zeichen holen
		call stabi     ;Startbit ausgeben
		mvi  c,8h      ;Bit-Zaehler
otlp:  	rrc            ;LSB zuerst ausgeben
		ana  a         ;Pegelabfrage
		jm   high_      ;Bit ist HIGH: JMP
low:   	call stabi     ;LOW-Bit ausgeben
		jmp  outcte
high_:  	call stobi     ;HIGH-Bit ausgeben
outcte:	dcr  c
		jnz  otlp
		call stobi     ;Stopbit ausgeben
		ret
;
stabi: 	push psw
		mvi  a,0c0h    ;SOD auf HIGH bringen
		jmp  bitcte_
stobi: 	push psw
		mvi  a,40h     ;SOD auf LOW bringen
bitcte_:	sim
		call dely2     ;Wartezeit
		pop  psw
		ret
;
;      CPUIN     Eingabe ins REG A ueber die CPU-
;                Schnittstelle SID
;
cpuin:	rim            ;SID abfragen
		ana  a
		jp   cpuin     ;SID auf LOW: JMP
		push d
		mvi  d,8h      ;Bit-Zaehler
		mvi  e,0h      ;Schieberegister
		call dely1     ;halbe Bit-Dauer warten
inlp: 	call dely2     ;Mitte Datenbit
		rim
		ani  80h       ;MSB maskieren
		ora  e         ;verknuepfen
		rrc
		mov  e,a       ;ins Schieberegister
		dcr  d
		jnz  inlp
		rlc            ;Ergebnis in REG A
		push psw
		call dely2     ;Mitte Stopbit
		pop  psw
		nop            ;87er:CMA (rueckinvertieren)
		pop  d
		ret
;
		DS	7h		; Platzhalter fuer dely2
					; bereits im teil 2 enthalten
;dely2:	call dely1
;		call dely1
;		ret
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Ansteuerung des Thermodruckers
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
fcthex:	lhld adres     ;HEX-Dump
		xchg           ;D&E: Endadresse
		lhld srcbeg    ;Anfangsadresse
		mvi  a,40h
		out  fdctrl    ;Hilfs-Adre~latch
		call prthex
		xra  a
		out  fdctrl
		inr  a
		ret
;
fctasc:	lhld adres     ;ASCII-Ausdruck
		shld srcend    ;Zeilenzahl
		lhld srcbeg    ;String-Anfang
		mvi  a,40h
		out  fdctrl    ;Hilfs-Adre~latch
		call prtasc
		xra  a
		out  fdctrl
		inr  a
		ret
;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                   ;
;      STP/RAM  Einzelschritt- und RAM-Funktionen   ;
;                                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;   Stand:   Do,19.02.87;12:49:06h
;
;   Druck:
;
	org    0a2ah
;ofs    0a2ah
;
;
;      Drei Betriebsarten m|glich:
;
;      1 Einzel-Byte-Betrieb
;        (Hardware-Einzel-Wort-Betrieb mit
;        Einzelschritt-Modul; RAM oder ROM)
;      2 Breakpoint-Betrieb
;        (Software-Haltepunkt im RAM setzen)
;      3 Einzelschritt-Betrieb
;        (Software-Single-Step im RAM)
;
;      Bei den Betriebsarten 2 und 3 wird der
;      Inhalt des CPU-FLAG-Registers in der
;      LED-Zeile angezeigt:
;
;      Bit   7   6   5   4   3   2   1   0
;      FLAG  S   Z   x   c   0   P   x   C
;           (Sign, Zero, Parity und Carry;
;            c=Aux.CARRY nicht testbar,
;            x ist undefiniert)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Einzel-Byte-Betrieb mit Einzelschritt-Modul
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
fctbyt:	lxi  h,rambeg  ;Start bei 2800h
		mvi  a,40h     ;BYT-Flipflop setzen
		out  stpff
		pchl           ;Byte-Mode starten
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Breakpoint-Betrieb im RAM
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
fctbrk:	lxi  h,warm    ;Warmstart-Adresse
brkmod:	call brkini    ;Breakpoints vorbereiten
		lhld adres     ;Break-Adresse
		shld brkpt1    ;retten
		mov  a,m       ;OpCode holen
		sta  opcod1    ;und retten
		mvi  m,0d7h    ;"RST2" einsetzen
bresto:	lda  regf      ;RESTOR fuer FCT B
		mov  c,a       ;REG C: CPU-FLAGs
		push b
		pop  psw       ;CPU-FLAGs setzen
		lhld regs      ;User-Stack-Pointer
		mov  a,l       ;sortieren
		mov  l,h       ;(H&L vertauschen)
		mov  h,a
		sphl           ;in CPU-Stack-Pointer
		lxi  h,regb    ;Pointer auf REG-Buffer
		mov  b,m       ;REG B laden
		inx  h
		mov  c,m       ;REG C laden
		inx  h
		mov  d,m       ;REG D laden
		inx  h
		mov  e,m       ;REG E laden
		lhld srcbeg    ;Startadresse holen
		push h         ;und im Stack ablegen
		lhld regh      ;Daten fuer H&L holen
		mov  a,l       ;und sortieren
		mov  l,h       ;(H&L vertauschen)
		mov  h,a
		lda  rega      ;REG A laden
		ret            ;Start bis zum Breakpoint
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Einzelschritt-Betrieb im RAM
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
fctstp:	lxi  h,3h      ;Warmstart-Adresse
stpmod:	call brkini    ;Breakpoints vorbereiten
		ret
;
;      BRKINI    Breakpoints vorbereiten (STP-FLAG
;                setzen, REG F l|schen, CTE-Adresse
;                im Hilfsbuffer HIBUF ablegen)
;
brkini:	shld hbuf2     ;CTE-Adresse (Warmstart)
		mvi  a,0c3h    ;"JMP"
		sta  hibuf2    ;CTE-Adresse fuer HIBUF
		lda  modflg    ;Mode-FLAG holen
		ani  0f7h      ;FCT-FLAG l|schen
		ori  80h       ;STP-Bit setzen
		sta  modflg
		xra  a         ;REG F l|schen
		sta  regf
		lxi  h,0h      ;Breakpoint-Adr.l|schen
		shld brkpt1
		shld brkpt2
		lxi  h,dig7
		mvi  m,0h
		dcx  h
		mvi  m,50h     ; "r"
		dcx  h
		mvi  m,1ch     ; "u"
		dcx  h
		mvi  m,54h     ; "n"
		ret
;
;      REST1     ersetzt den naechstfolgenden OpCode
;                durch einen Breakpoint (RST2=D7h)
;
rest1: 	lhld adres     ;aktuelle Adresse holen
		call rest11    ;Adr.des naechsten OpCodes
		mov  a,m
		sta  opcod1    ;OpCode retten
		mvi  m,0d7h    ;"RST2" einsetzen
;
;      REST12    setzt bei den Befehlen PCHL, RST,
;                RET, JMP bzw. CALL einen zweiten
;                Breakpoint an der m|glichen Ziel-
;                adresse ein (ebenfalls RST2=D7h)
;
rest12:	ldax d         ;OpCode erneut holen
		cpi  0e9h      ;ist es PCHL?
		jnz  rstct2    ;nein: JMP
		lhld regh      ;Zieladresse in H&L
		mov  a,l       ;H&L vertauschen
		mov  l,h
		mov  h,a
		jmp  rstend
rstct2:	call rstbef    ;ist es RSTn?
		jz   rstct3    ;nein: JMP
		ldax d
		ani  038h      ;Ziel-Adre~bits maskieren
		inr  a         ;Adresse hinter "C3"
		mov  l,a
		mvi  h,0h      ;H&L: 01...39h
		mov  e,m       ;Zieladresse (lower)
		inx  h
		mov  d,m       ;Zieladresse (upper)
		xchg           ;H&L: RST-Sprungziel
		jmp  rstend
rstct3:	call retbef    ;ist es RET?
		jz   rstct4    ;nein: JMP
		lhld regs      ;User-Stack-Pointer
		mov  a,l       ;sortieren
		mov  l,h       ;(H&L vertauschen)
		mov  h,a
		mov  e,m       ;Stack auslesen
		inx  h
		mov  d,m       ;D&E: RET-Adresse
		xchg
		jmp  rstend
rstct4:	call jmpbef    ;ist es JMP/CALL?
		rz             ;nein: RET
		inx  d         ;Zieladresse holen
		ldax d
		mov  l,a       ;untere Adre~haelfte
		inx  d
		ldax d
		mov  h,a
rstend:	shld brkpt2    ;Break-Adr.#2 retten
		mov  a,m       ;OpCode holen
		sta  opcod2    ;und retten
		mvi  m,0d7h    ;"RST2" einsetzen
		ret
;
;      REST11  ermittelt die Adresse des naechsten
;              OpCodes (H&L beim Einsprung: Adresse
;              des aktuellen OpCodes)
;
rest11:	push h
		xchg           ;D&E: Adr.aktueller OpCode
		mvi  b,3h      ;Schrittweite Dreiwort-Befehl
		call drwbef    ;ist es Dreiwort-Befehl?
		jnz  rstcte    ;ja: JMP
		dcr  b         ;Schrittweite Zweiwort-Befehl
		call zwwbef    ;ist es Zweiwort-Befehl?
		jnz  rstcte    ;ja: JMP
		dcr  b         ;Schrittweite Einwort-Befehl
rstcte:	mov  c,b
		mvi  b,0h      ;B&C: 1, 2, oder 3
		pop  h
		dad  b         ;H&L: naechster OpCode
		shld brkpt1
		ret
;
;      untersucht OpCode-Typ (Adresse steht
;      beim Einsprung in D&E); ist er in der
;      betreffenden Tabelle enthalten, ist
;      das ZERO-FLAG beim Ruecksprung off
;      (Z=0); andernfalls ist Z=1.
;
zwwbef:	lxi  h,op2tab  ;Zweiwort-Befehl?
		mvi  c,12h     ;Tabellenlaenge 18d
		jmp  oplop
drwbef:	lxi  h,op3tab  ;Dreiwort-Befehl?
		mvi  c,1ah     ;Tabellenlaenge 26d
		jmp  oplop
jmpbef:	lxi  h,jmptab  ;Jump/Call-Befehl?
		mvi  c,12h     ;Tabellenlaenge 18d
		jmp  oplop
retbef:	lxi  h,retab   ;Return-Befehl?
		mvi  c,9h      ;Tabellenlaenge 9d
		jmp  oplop
rstbef:	lxi  h,rstab   ;Restart-Befehl?
		mvi  c,8h      ;Tabellenlaenge 8d
oplop: 	ldax d         ;OpCode holen
		cmp  m         ;(REG A)=Tab.wert?
		jz   opend     ;ja: JMP
		inx  h
		dcr  c
		jnz  oplop
opend: 	xra  a         ;REG A l|schen
		cmp  c         ;Tab. durchlaufen?
		ret            ;ja: Z=1
;
;      REST2     ersetzt Breakpoints durch die ur-
;                spruenglichen OpCodes
;
rest2: 	shld hibuf     ;H&L voruebergehend retten
		sta  hbuf      ;REG A ebenfalls
		pop  h         ;gleicht RST-Interrupt aus
		dcx  h         ;wegen Einwort-Interrupt
		shld adres     ;aktualiserte Adresse
		lhld brkpt1    ;Break-Adresse #1 holen
		lda  opcod1    ;und geretteten OpCode #1
		mov  m,a       ;wieder einsetzen
		lhld brkpt2    ;Break-Adresse #2 holen
		lda  opcod2    ;und geretteten OpCode #2
		mov  m,a       ;wieder einsetzen
		lda  hbuf      ;REG A laden
		lhld hibuf     ;H&L ebenfalls
		call save      ;saemtliche REGs in RAM-Buffer
		lxi  h,0h      ;Break-Adressen l|schen
		shld brkpt1
		shld brkpt2
		push h         ;Eintragungen im User-Stack
		push h         ;l|schen
		pop  h
		pop  h
		lxi  sp,stack  ;Monitor-Stack!
		jmp  hibuf2    ;zum Warmstart
;
;;;
;
;      Befehlstabellen
;
;;;
;
op2tab:	db   06h       ;MVI B    Zweiwort-Befehle
		db   0eh       ;MVI C
		db   16h       ;MVI D
		db   1eh       ;MVI E
		db   26h       ;MVI H
		db   2eh       ;MVI L
		db   36h       ;MVI m
		db   3eh       ;MVI A
;
		db   0c6h      ;ADI
		db   0ceh      ;ACI
		db   0d6h      ;SUI
		db   0deh      ;SBI
		db   0e6h      ;ANI
		db   0eeh      ;XRI
		db   0f6h      ;ORI
		db   0feh      ;CPI
;
		db   0d3h      ;OUT
		db   0dbh      ;IN
;
op3tab:	db   01h       ;LXI B    Dreiwort-Befehle
		db   11h       ;LXI D
		db   21h       ;LXI H
		db   22h       ;SHLD
		db   2ah       ;LHLD
		db   31h       ;LXI SP
		db   32h       ;STA
		db   3ah       ;LDA
;
jmptab:	db   0c2h      ;JNZ      Sprungbefehle
		db   0c3h      ;JMP uncond.
		db   0cah      ;JZ
		db   0d2h      ;JNC
		db   0dah      ;JC
		db   0e2h      ;JPO
		db   0eah      ;JPE
		db   0f2h      ;JP
		db   0fah      ;JM
;
caltab:	db   0c4h      ;CNZ      Aufruf v.Unt.progr.
		db   0cch      ;CZ
		db   0cdh      ;CALL uncond.
		db   0d4h      ;CNC
		db   0dch      ;CC
		db   0e4h      ;CPO
		db   0ech      ;CPE
		db   0f4h      ;CP
		db   0fch      ;CM
;
retab: 	db   0c0h      ;RNZ      Rueckspr.v.Unt.progr.
		db   0c8h      ;RZ
		db   0c9h      ;RET uncond.
		db   0d0h      ;RNC
		db   0d8h      ;RC
		db   0e0h      ;RPO
		db   0e8h      ;RPE
		db   0f0h      ;RP
		db   0f8h      ;RM
;
rstab: 	db   0c7h      ;RST0
		db   0cfh      ;RST1
		db   0d7h      ;RST2
		db   0dfh      ;RST3
		db   0e7h      ;RST4
		db   0efh      ;RST5
		db   0f7h      ;RST6
		db   0ffh      ;RST7
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      RAM-Verwaltung
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      EXCHAN    kopiert Speicherbereiche um und
;                pa~t dabei diejenigen Adressen
;                an, die im kopierten Block
;                liegen.
;
fctxch:	lhld adres
		shld dstbeg    ;Zieladresse
		call excha_
		jmp  fctret    ;Daten-Mode bei Zieladresse
		nop
;
excha_:	lhld srcend    ;save Params
		push h
		lhld srcbeg
		push h
		lhld dstbeg
		push h
		call copy_     ;erst umkopieren
		pop  d
		pop  h
		shld srcbeg    ;restore SRCBEG
		push psw
		xchg
		call diffhd    ;H&L=DSTBEG-SRCBEG (Adre~diff.)
		xchg
		pop  psw
		pop  h
		shld srcend    ;restore SRCEND
		dad  d         ;plus Adre~differenz
		shld brkpt2    ;Ende im Zielbereich (Endabfr.)
		rz             ;Fehler bei COPY
exlop: 	lhld brkpt1    ;aktueller OpCode
		shld hibuf     ;Adresse retten
		call rest11    ;naechst.OpCode n.BRKPT1
		xchg           ;D&E: Adr.naechster OpCode
		lhld brkpt2    ;H&L: Endadresse Zielbereich
		call cmpnd     ;Ende?
		jnc  copyok    ;ja: dann Ruecksprung
		lhld hibuf     ;Adresse zurueckholen
		xchg
		call drwbef    ;ist es Dreiwort-Befehl?
		cnz  modify    ;ja: CALL
		jmp  exlop
;
;      MODIFY    modifiziert die in einem Dreiwort-
;                Befehl enthaltene Absolutadresse,
;                wenn diese im Bereich SRCBEG...
;                SRCEND liegt
;
modify:	inx  d         ;D&E: Adr. des akt. OpCodes
		ldax d         ;Absolutadresse (lower)
		mov  l,a
		inx  d
		ldax d         ;Absolutadresse (upper)
		mov  h,a       ;H&L: Absolutadresse
		call cmpend    ;Vergleich mit SRCEND
		rnc            ;gr|~er als SRCEND: RET
		lhld srcbeg    ;D&E: Absolutadresse
		call cmpnd     ;Vergleich mit SRCBEG
		jz   mod       ;Gleichh.: auch modifiz.!
		rc             ;kleiner als SRCBEG: RET
mod:   	lhld dstend    ;Adre~differenz
		dad  d         ;zur Absol.adr. addieren
		xchg           ;D&E: modifiz. Adresse
		lhld hibuf     ;aktueller OpCode
		inx  h         ;zweites Byte im Befehl
		mov  m,e       ;modifiz.Adr. (lower)
		inx  h         ;drittes Byte im Befehl
		mov  m,d       ;modifiz.Adr. (upper)
		ret
;
;
;      INSERT    Fuegt an der aktuellen RAM-Adresse
;                NOPs (=00) ein (RAM-Ende beachten!)
;                hierbei keine Adre~anpassung!
;
fctins:	lda  adres     ;Anzahl holen
		push psw
		mov  e,a
		mvi  d,0h
		lhld srcbeg
		push h
		dad  d         ;Zieladresse
		shld dstbeg
		call fctsub    ;umkopieren
		pop  h
		pop  psw
		push h
inslop:	mvi  m,0h      ;NOP einfuegen
		inx  h
		dcr  a
		jnz  inslop
		pop  h
		jmp  datset    ;Daten-Mode
;
;
;      DELETE    L|scht RAM-Zellen und rueckt
;                RAM-Inhalt nach (bis RAM-Ende)
;                hierbei keine Adre~anpassung!
;
fctdel:	lda  adres     ;Anzahl holen
		mov  e,a
		mvi  d,0h
		lhld srcbeg
		shld dstbeg
		push h
		dad  d         ;Startadresse
		shld srcbeg
		call fctsub    ;umkopieren
		pop  h
		jmp  datset    ;Daten-Mode
;
fctsub:	lxi  h,ramend
		shld srcend
		jmp  copy_
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      RAM-Bereiche verschieben
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
fctcpy:	lhld adres     ;dritter Parameter
		shld dstbeg
		call copy_     ;kopieren ausfuehren
fctret:	lhld dstbeg    ;Zieladresse holen
		jmp  datset    ;und Daten-Mode einstellen
;
;      COPY      kopiert Speicherbereiche
;
;      SRCBEG:   Anfang Ursprungsbereich
;      SRCEND:   Ende Ursprungsbereich
;      DSTBEG:   Anfang Zielbereich
;
copy_: 	lhld srcbeg
		xchg
		lhld srcend
		call diffhd    ;H&L=SRCEND-SRCBEG
		jnc  cpycte    ;Differenz positiv
		xra  a
		mvi  a,16h     ;SRCBEG > SRCEND
		ret
cpycte:	push h         ;save Blocklaenge
		lhld dstbeg
		xchg
		lhld srcbeg
		call diffhd    ;H&L=SRCBEG-DSTBEG
		pop  d
		jnc  cpypls    ;Start Abwaertskopieren
;
		lhld dstbeg
		dad  d
		shld dstbeg    ;DSTBEG=DSTBEG+Blocklaenge
		lhld srcbeg
		xchg
		lhld srcend
		shld srcbeg    ;bei SRCEND anfangen
		xchg
		shld srcend    ;bei SRCBEG aufh|ren
		lxi  b,0ffffh  ;-1d
		jmp  cpysub    ;Start Aufwaertskopieren
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      RAM mit Konstante auffuellen
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
fctfil:	lda  adres     ;Konstante holen
		sta  dstbeg
		call fill_     ;Funktion ausfuehren
		lhld srcbeg    ;Startdresse holen
		jmp  datset    ;und Daten-Mode einstellen
;
;      	FILL    fuellt RAM mit Konstante auf
;
;      	SRCBEG: Anfangsadresse
;      	SRCEND: Endadresse
;      	DSTBEG: Konstante
;
		jmp  kill
fill_: 	lhld srcbeg
		xchg
		lhld srcend
		call diffhd
		jnc  filcte
		xra  a
		mvi  a,17h
		jmp  cpyer     ;SRCEND < SRCBEG
filcte:	mov  a,l
		ora  h
		mvi  a,18h
		jz   cpyer     ;SRCEND = SRCBEG
		lhld srcbeg
		lda  dstbeg
		mov  m,a
		inx  h
		shld dstbeg
		lhld srcend
		dcx  h
		shld srcend
		jmp  cpypls
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Bytes im RAM durch andere ersetzen
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      KILL    ersetzt Bytes durch andere
;
;      SRCBEG: Anfangsadresse
;      SRCEND: Endadresse
;      DSTBEG: HI:zu ersetzende Bytes
;              LO:Ersatz-Byte
;
kill:  	lhld dstbeg
		push h
		pop  b         ;C:Ersatz-,B:zu ersetzendes Z.
kilop: 	lhld srcbeg
		mov  a,m
		cmp  b
		jnz  kilnxt
		mov  m,c       ;neues Byte einschreiben
		mov  a,m
		cmp  c
		jnz  cpyerr    ;Einschreiben fehlerhaft
kilnxt:	call cmpend
		jz   copyok    ;Ende erreicht
		lhld srcbeg
		inx  h
		shld srcbeg
		jmp  kilop
;
;
;;;
;
;      Kopier-Routine
;
;;;
;
cpypls:	lxi  b,1h      ;Adr.hochzaehlen
cpysub:	lhld dstbeg
		xchg
		lhld srcbeg
cpysb: 	mov  a,m       ;wird mind.1x ausgefuehrt!
		stax d
		ldax d
		cmp  m
		jnz  cpyerr    ;Einschreiben fehlerhaft
		call cmpend
		jz   copyok
		call adrinr    ;B&C: +1 oder -1!
		jmp  cpysub
copyok:	xra  a
		inr  a         ;ZERO off
		ret
cpyerr:	xra  a         ;ZERO on
		mvi  a,19h     ;Einschreiben fehlerhaft
cpyer: 	lhld srcbeg    ;REVISE vorbereiten
		shld srcend
		xchg
		shld dstbeg
		ret
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Verwaltungs-Unterprogramme
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
maxra_:	lxi  h,8000h   ;A&B:h|chste RAM-Adr.ab 8000h
		lxi  d,0fdh    ;Schrittweite (random)
		mvi  b,"r"     ;Konstante
maxlop:	dad  d
		mov  c,m       ;save RAM
		mov  m,b
		mov  a,m
		mov  m,c       ;rest.RAM
		cmp  b
		jz   maxlop    ;Einschreiben erfolgreich
		mov  a,h
		dcr  a
		mvi  b,0ffh
		ret
;
;
diffhd:	shld brkpt1
		mov  a,l
		sub  e
		mov  l,a
		mov  a,h
		sbb  d
		mov  h,a
		shld dstend    ;H&L=Diff(H&L-D&E);->DSTEND
		ret
;
;
adrinr:	lhld srcbeg    ;lfd.Adr.erh|hen/erniedrigen
		dad  b         ;+1 oder-1!
		shld srcbeg
		lhld dstbeg
		dad  b
		shld dstbeg
		ret
;
;
cmpend:	xchg           ;H&L < (SRCEND): CY on
		lhld srcend
cmpnd: 	mov  a,d
		cmp  h
		rc
		rnz
		mov  a,e
		cmp  l
		ret            ;H&L = (SRCEND): ZERO on

;
;----------------------------------------------------------
;
; Ende Monitor rot Routinen
;
; ab hier Textmeldungen und ASCII-Tabell aus Monitor gelb
; dort als org    menue0m [0d82h]
;----------------------------------------------------------
;
; Systemram
;
;      MAIN: HEXMO, STP/RAM, EZU/DSP, Peripherie
;
;
	org    2f8ch          ;PROMO-Buffer
;
;      Interrupt-Sprungverteiler
;
intrap:	ds   3h
int3: 	ds   3h
int4:  	ds   3h
int5:  	ds   3h
int55: 	ds   3h
int6:  	ds   3h
int65: 	ds   3h
int7:  	ds   3h
int75: 	ds   3h
;
;      	Hilfszellen fuer den Monitor
;
rega:  	ds   1h        ; REG A
regb:  	ds   1h        ; REG B
regc:  	ds   1h        ; REG C
regd:  	ds   1h        ; REG D
rege:  	ds   1h        ; REG E
regf:  	ds   1h        ; FLAGs
regh:  	ds   1h        ; REG H
regl:  	ds   1h        ; REG L
regs:  	ds   1h        ; Stack-Pointer (HI)
regp:  	ds   1h        ; Stack-Pointer (LO)
		ds   1h        ; frei
;
dspbuf:	ds   1h        ; LED-Zeile
dig1:  	ds   1h        ; Digit 1
dig2:  	ds   1h        ; Digit 2
dig3:  	ds   1h        ; Digit 3
dig4:  	ds   1h        ; Digit 4
dig5:  	ds   1h        ; Digit 5
dig6:  	ds   1h        ; Digit 6
dig7:  	ds   1h        ; Digit 7
;
data:  	ds   1h        ; Daten
adres: 	ds   1h        ; Adresse
adrhi: 	ds   1h
modflg:	ds   1h        ; Mode-FLAG
fctbuf:	ds   1h        ; FCT-Nr.
srcbeg:	ds   2h        ; Par.1
srcend:	ds   2h        ; Par.2
dstbeg:	ds   2h        ; Par.3
dstend:	ds   2h        ; Par.4
regnr: 	ds   1h        ; REG-Nr.
ioflg: 	ds   1h        ; I/O-Param.
nxtbuf:	ds   1h        ; FCT-Eingaben
maxbuf:	ds   1h        ; max. Anz.
keybyt:	ds   1h        ; KEY true./.invers
ciflag:	ds   1h        ; FLAG f. CI-Routine
hibuf: 	ds   2h        ; I/O-Hilfsroutine
hbuf:  	ds   1h
;
hibuf2:	ds   1h        ; CTE-Sprungziel
hbuf2: 	ds   2h
brkpt1:	ds   2h        ; BRK-Adresse #1
opcod1:	ds   1h        ; OpCode-Buf. #1
brkpt2:	ds   2h        ; BRK-Adresse #2
opcod2:	ds   2h        ; OpCode-Buf. #2
;
;
;
;      BCD-Uhrzeit-Buffer
;
	org    uhrbuf
;
seke:  	ds   1h
sekz:  	ds   1h
mine:  	ds   1h
minz:  	ds   1h
stde:  	ds   1h
stdz:  	ds   1h
klte:  	ds   1h
kltz:  	ds   1h
wtgl:  	ds   1h
wtgu:  	ds   1h
mone:  	ds   1h
monz:  	ds   1h
jare:  	ds   1h
jarz:  	ds   1h
;
;      ASCII-Uhrzeit-Buffer
;
ascbuf:	ds   15h
;

;
end
;
